@@ -23,7 +23,8 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
                                                      !autopilot work
   USE core,                     ONLY : rhoc
   USE uspp_param,               ONLY : nhm, nh
-  USE uspp,                     ONLY : nkb, vkb, vkb_d, becsum, deeq, okvan, nlcc_any
+  USE pseudo_base,              ONLY : vkb_d
+  USE uspp,                     ONLY : nkb, vkb, becsum, deeq, okvan, nlcc_any
   USE energies,                 ONLY : eht, epseu, exc, etot, eself, enl, &
                                        ekin, atot, entropy, egrand, enthal, &
                                        ekincm, print_energies
@@ -97,6 +98,9 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
                                        irb, taub, eigrb, rhog, rhos, &
                                        rhor, bephi, becp_bgrp, nfi, idesc, &
                                        drhor, drhog, bec_bgrp, dbec, bec_d, iabox, nabox
+#if defined (__CUDA)
+USE cp_main_variables,        ONLY : eigr_d
+#endif
   USE autopilot,                ONLY : event_step, event_index, &
                                        max_event_step, restart_p
   USE cell_base,                ONLY : s_to_r, r_to_s
@@ -574,7 +578,6 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
          IF ( tortho ) THEN
 #if defined (__CUDA)
             CALL updatc( ccc, lambda, phi, bephi, becp_bgrp, bec_d, cm_d, idesc )
-            CALL dev_memcpy( bec_bgrp, bec_d )
             CALL dev_memcpy( cm_bgrp, cm_d )
 #else
             CALL updatc( ccc, lambda, phi, bephi, becp_bgrp, bec_bgrp, cm_bgrp, idesc )
@@ -586,16 +589,34 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
            cm_bgrp(:,iupdwn(2):nbsp)   =     cm_bgrp(:,1:nupdwn(2))
            !phi(:,iupdwn(2):nbsp)       =    phi(:,1:nupdwn(2))
            CALL dev_memcpy(phi(:,iupdwn(2):), phi, [1, ubound(phi)], 1, [1, nbsp])
+           CALL dev_memcpy(cm_d(:,iupdwn(2):), cm_d, [1, ubound(cm_d)], 1, [1, nbsp])
            lambda(:,:, 2) = lambda(:,:, 1)
          ENDIF
          !
          ! the following compute only on NC pseudo components
+#if defined (__CUDA)
+         CALL dev_memcpy( eigr_d, eigr )
+         !CALL dev_memcpy( cm_d, cm_bgrp )
+         CALL calbec( nbsp_bgrp, vkb_d, cm_d, bec_d, 1 )
+         !CALL dev_memcpy( vkb, vkb_d )
+         !CALL dev_memcpy( bec_bgrp, bec_d )
+#else
          CALL calbec( nbsp_bgrp, vkb, cm_bgrp, bec_bgrp, 1 ) 
+#endif
          !
          IF ( tpre ) THEN
+#if defined (__CUDA)
+           CALL caldbec_bgrp( eigr_d, cm_d, dbec, idesc )
+#else
            CALL caldbec_bgrp( eigr, cm_bgrp, dbec, idesc )
+#endif
          END IF
          !
+#if defined (__CUDA)
+        CALL dev_memcpy( vkb, vkb_d )
+        CALL dev_memcpy( bec_bgrp, bec_d )
+#endif
+         !
          IF ( iverbosity > 1 ) CALL dotcsc( vkb, cm_bgrp, ngw, nbsp_bgrp )
          !
        END IF
@@ -1126,6 +1147,7 @@ SUBROUTINE terminate_run()
   CALL print_clock( 'new_ns' )
   CALL print_clock( 'strucf' )
   CALL print_clock( 'calbec' )
+  CALL print_clock( 'caldbec_bgrp' )
   CALL print_clock( 'exch_corr' )
 !==============================================================
   IF (ts_vdw) THEN
