@@ -29,7 +29,7 @@
       use ions_base,            only : nat
       USE recvec_subs,          ONLY : ggen, ggens
       USE gvect,                ONLY : mill_g, eigts1,eigts2,eigts3, g, gg, &
-                                       g_d, ecutrho, gcutm, gvect_init, mill,&
+                                       ecutrho, gcutm, gvect_init, mill,&
                                        ig_l2g, gstart, ngm, ngm_g, gshells
       use gvecs,                only : gcutms, gvecs_init, ngms
       use gvecw,                only : gkcut, gvecw_init, g2kin_init
@@ -184,11 +184,11 @@
         CALL ggens( dffts, gamma_only, at, g, gg, mill, gcutms, ngms )
         !
       END IF
-
-#if defined (__CUDA)
-      g_d = g
-#endif
-
+!NOTE g and mill already allocate in the device they are initialized below. 
+!$acc data present(g, mill) 
+!$acc update device(g,mill) 
+!$acc end data 
+      !
       CALL gshells (.TRUE.)
       !
       ! ... allocate and generate (modified) kinetic energy
@@ -394,7 +394,7 @@
       USE cell_base,             ONLY : at, bg, omega, alat, tpiba2, &
                                         cell_base_reinit
       USE gvecw,                 ONLY : g2kin_init
-      USE gvect,                 ONLY : g, g_d, gg, ngm, mill
+      USE gvect,                 ONLY : g, gg, ngm, mill
       USE fft_base,              ONLY : dfftp, dfftb
       USE small_box,             ONLY : small_box_set
       USE smallbox_subs,         ONLY : gcalb
@@ -406,7 +406,7 @@
       INTEGER,  INTENT(IN) :: iverbosity
       !
       REAL(DP) :: rat1, rat2, rat3
-      INTEGER :: ig
+      INTEGER :: ig, dfftp_ngm
       !
       !WRITE( stdout, "(4x,'h from newinit')" )
       !do i=1,3
@@ -419,13 +419,14 @@
       !
       !  re-calculate G-vectors and kinetic energy
       !
-      do ig = 1, dfftp%ngm
+      dfftp_ngm = dfftp%ngm 
+!$acc parallel loop present(g, mill) copyin(bg) copyout(gg)  
+      do ig = 1, dfftp_ngm
          g(:,ig)= mill(1,ig)*bg(:,1) + mill(2,ig)*bg(:,2) + mill(3,ig)*bg(:,3)
-         gg(ig)=g(1,ig)**2 + g(2,ig)**2 + g(3,ig)**2
+         gg(ig)=g(1,ig)**2 + g(2,ig)**2 + g(3,ig)**2 
       enddo
-#if defined (__CUDA)
-      g_d = g
-#endif
+!$acc end parallel loop 
+!$acc update host(g) 
       !
       call g2kin_init ( gg, tpiba2 )
       !
