@@ -635,9 +635,7 @@
     !! degaussw0 = (ismear-1) * delta_smear + degaussw
     REAL(KIND = DP) :: inv_degaussw0
     !! Inverse degaussw0 for efficiency reasons
-    REAL(KIND = DP) :: eptemp0
-    !!eptemp0   = (ismear-1) * delta_smear + eptem
-    REAL(KIND = DP) :: inv_eptemp0
+    REAL(KIND = DP) :: inv_eptemp
     !! Inverse of temperature define for efficiency reasons
     REAL(KIND = DP) :: lambda_tot
     !! Integrated lambda function
@@ -720,12 +718,11 @@
       DO ismear = 1, nsmear
         !
         degaussw0 = (ismear - 1) * delta_smear + degaussw
-        eptemp0   = (ismear - 1) * delta_smear + gtemp(itemp)
         !
         ! SP: Multiplication is faster than division ==> Important if called a lot
         !     in inner loops
         inv_degaussw0 = one / degaussw0
-        inv_eptemp0   = one / eptemp0
+        inv_eptemp   = one / gtemp(itemp)
         !
         ! Fermi level and corresponding DOS
         !
@@ -771,7 +768,7 @@
           ! we may implement the approximation to the angle between k and k+q
           ! vectors also listed in Grimvall
           !
-          IF (vme) THEN
+          IF (vme == 'wannier') THEN
             DO ibnd = 1, nbndfst
               DO jbnd = 1, nbndfst
                 !
@@ -788,11 +785,11 @@
             DO ibnd = 1, nbndfst
               DO jbnd = 1, nbndfst
                 !
-                ! v_(k,i) = 1/m <ki|p|ki> = 2 * dmef (:, i,i,k)
+                ! v_(k,i) = 1/m <ki|p|ki> = dmef (:, i,i,k)
                 ! 1/m  = 2 in Rydberg atomic units
                 !
-                vkk(:, ibnd) = 2.0 * REAL(dmef(:, ibndmin - 1 + ibnd, ibndmin - 1 + ibnd, ikk))
-                vkq(:, jbnd) = 2.0 * REAL(dmef(:, ibndmin - 1 + jbnd, ibndmin - 1 + jbnd, ikq))
+                vkk(:, ibnd) = REAL(dmef(:, ibndmin - 1 + ibnd, ibndmin - 1 + ibnd, ikk))
+                vkq(:, jbnd) = REAL(dmef(:, ibndmin - 1 + jbnd, ibndmin - 1 + jbnd, ikq))
                 IF (ABS(vkk(1, ibnd)**two + vkk(2, ibnd)**two + vkk(3, ibnd)**two) > eps4) &
                   coskkq(ibnd, jbnd) = DDOT(3, vkk(:, ibnd), 1, vkq(:, jbnd), 1)  / &
                                        DDOT(3, vkk(:, ibnd), 1, vkk(:, ibnd), 1)
@@ -815,7 +812,7 @@
                 IF (delta_approx) THEN
                   w0g1 = w0gauss(ekk * inv_degaussw0, 0) * inv_degaussw0
                 ELSE
-                  wgkk = wgauss(-ekk * inv_eptemp0, -99)
+                  wgkk = wgauss(-ekk * inv_eptemp, -99)
                 ENDIF
                 !
                 DO jbnd = 1, nbndfst
@@ -845,7 +842,7 @@
                     !
                   ELSE
                     !
-                    wgkq = wgauss(-ekq * inv_eptemp0, -99)
+                    wgkq = wgauss(-ekq * inv_eptemp, -99)
                     !
                     ! = k-point weight * [f(E_k) - f(E_k+q)] / [E_k+q - E_k - w_q + id]
                     ! This is the imaginary part of the phonon self-energy, sans
@@ -1264,8 +1261,9 @@
                     ENDIF
                   ELSE
                     IF (ABS(ekk - ekk1) > eps8) THEN
-                      dipole = REAL(      dmef(1, ibndmin - 1 + jbnd, ibndmin - 1 + ibnd, ikk) *  &
-                                    CONJG(dmef(1, ibndmin - 1 + jbnd, ibndmin - 1 + ibnd, ikk)) / &
+                      ! TODO: Check the expression to confirm that division by 2 is correct.
+                      dipole = REAL(      dmef(1, ibndmin - 1 + jbnd, ibndmin - 1 + ibnd, ikk) / 2.d0 *  &
+                                    CONJG(dmef(1, ibndmin - 1 + jbnd, ibndmin - 1 + ibnd, ikk) / 2.d0) / &
                                     ((ekk1 - ekk)**two + sq_degaussw))
                     ELSE
                       dipole = zero
