@@ -13,29 +13,39 @@
 SUBROUTINE laxlib_cdiaghg( n, m, h, s, ldh, e, v, me_bgrp, root_bgrp, intra_bgrp_comm )
   !----------------------------------------------------------------------------
   !
-  ! ... calculates eigenvalues and eigenvectors of the generalized problem
-  ! ... Hv=eSv, with H hermitean matrix, S overlap matrix.
-  ! ... On output both matrix are unchanged
-  !
-  ! ... LAPACK version - uses both ZHEGV and ZHEGVX
+  !! Called by diaghg interface.
+  !! Calculates eigenvalues and eigenvectors of the generalized problem.
+  !! Solve Hv = eSv, with H symmetric matrix, S overlap matrix.
+  !! complex matrices version.
+  !! On output both matrix are unchanged.
+  !!
+  !! LAPACK version - uses both ZHEGV and ZHEGVX
+  !!
   !
   USE laxlib_parallel_include
   IMPLICIT NONE
-  INCLUDE 'laxlib_kinds.fh'
+  include 'laxlib_kinds.fh'
   !
-  INTEGER, INTENT(IN) :: n, m, ldh
-    ! dimension of the matrix to be diagonalized
-    ! number of eigenstates to be calculate
-    ! leading dimension of h, as declared in the calling pgm unit
-  COMPLEX(DP), INTENT(INOUT) :: h(ldh,n), s(ldh,n)
-    ! actually intent(in) but compilers don't know and complain
-    ! matrix to be diagonalized
-    ! overlap matrix
+  INTEGER, INTENT(IN) :: n
+  !! dimension of the matrix to be diagonalized
+  INTEGER, INTENT(IN) :: m
+  !! number of eigenstates to be calculated
+  INTEGER, INTENT(IN) :: ldh
+  !! leading dimension of h, as declared in the calling pgm unit
+  COMPLEX(DP), INTENT(INOUT) :: h(ldh,n)
+  !! matrix to be diagonalized
+  COMPLEX(DP), INTENT(INOUT) :: s(ldh,n)
+  !! overlap matrix
   REAL(DP), INTENT(OUT) :: e(n)
-    ! eigenvalues
+  !! eigenvalues
   COMPLEX(DP), INTENT(OUT) :: v(ldh,m)
-    ! eigenvectors (column-wise)
-  INTEGER, INTENT(IN) :: me_bgrp, root_bgrp, intra_bgrp_comm
+  !! eigenvectors (column-wise)
+  INTEGER,  INTENT(IN)  :: me_bgrp
+  !! index of the processor within a band group
+  INTEGER,  INTENT(IN)  :: root_bgrp
+  !! index of the root processor within a band group
+  INTEGER,  INTENT(IN)  :: intra_bgrp_comm
+  !! intra band group communicator
   !
   INTEGER                  :: lwork, nb, mm, info, i, j
     ! mm = number of calculated eigenvectors
@@ -197,11 +207,14 @@ END SUBROUTINE laxlib_cdiaghg
 !----------------------------------------------------------------------------
 SUBROUTINE laxlib_cdiaghg_gpu( n, m, h_d, s_d, ldh, e_d, v_d, me_bgrp, root_bgrp, intra_bgrp_comm)
   !----------------------------------------------------------------------------
-  !
-  ! ... calculates eigenvalues and eigenvectors of the generalized problem
-  ! ... Hv=eSv, with H hermitean matrix, S overlap matrix.
-  ! ... On output both matrix are unchanged
-  !
+  !!
+  !! Called by diaghg interface.
+  !! Calculates eigenvalues and eigenvectors of the generalized problem
+  !! Solve Hv = eSv, with H symmetric matrix, S overlap matrix.
+  !! complex matrices version.
+  !! On output both matrix are unchanged.
+  !!
+  !! GPU VERSION.
   !
 #if defined(_OPENMP)
   USE omp_lib
@@ -231,25 +244,32 @@ SUBROUTINE laxlib_cdiaghg_gpu( n, m, h_d, s_d, ldh, e_d, v_d, me_bgrp, root_bgrp
 #endif
   !
   IMPLICIT NONE
-  INCLUDE 'laxlib_kinds.fh'
+  include 'laxlib_kinds.fh'
   !
-  INTEGER, INTENT(IN) :: n, m, ldh
-    ! dimension of the matrix to be diagonalized
-    ! number of eigenstates to be calculate
-    ! leading dimension of h, as declared in the calling pgm unit
-  COMPLEX(DP), INTENT(INOUT) :: h_d(ldh,n), s_d(ldh,n)
-    ! actually intent(in) but compilers don't know and complain
-    ! matrix to be diagonalized, allocated on the GPU
-    ! overlap matrix, allocated on the GPU
+  INTEGER, INTENT(IN) :: n
+  !! dimension of the matrix to be diagonalized
+  INTEGER, INTENT(IN) :: m
+  !! number of eigenstates to be calculated
+  INTEGER, INTENT(IN) :: ldh
+  !! leading dimension of h, as declared in the calling pgm unit
+  COMPLEX(DP), INTENT(INOUT) :: h_d(ldh,n)
+  !! matrix to be diagonalized, allocated on the GPU
+  COMPLEX(DP), INTENT(INOUT) :: s_d(ldh,n)
+  !! overlap matrix, allocated on the GPU
   REAL(DP), INTENT(OUT) :: e_d(n)
-    ! eigenvalues, , allocated on the GPU
+  !! eigenvalues, , allocated on the GPU
   COMPLEX(DP),  INTENT(OUT) :: v_d(ldh,n)
-    ! eigenvectors (column-wise), , allocated on the GPU
-    ! NB: the dimension of v_d this is different from cdiaghg !!
+  !! eigenvectors (column-wise), , allocated on the GPU
+  INTEGER,  INTENT(IN)  :: me_bgrp
+  !! index of the processor within a band group
+  INTEGER,  INTENT(IN)  :: root_bgrp
+  !! index of the root processor within a band group
+  INTEGER,  INTENT(IN)  :: intra_bgrp_comm
+  !! intra band group communicator
+  !
 #if defined(__CUDA)
     ATTRIBUTES(DEVICE) :: h_d, s_d, e_d, v_d
 #endif
-  INTEGER, INTENT(IN) :: me_bgrp, root_bgrp, intra_bgrp_comm
   !
   INTEGER              :: lwork, info
   !
@@ -506,40 +526,45 @@ END SUBROUTINE laxlib_cdiaghg_gpu
 SUBROUTINE laxlib_pcdiaghg( n, h, s, ldh, e, v, idesc )
   !----------------------------------------------------------------------------
   !
-  ! ... calculates eigenvalues and eigenvectors of the generalized problem
-  ! ... Hv=eSv, with H hermitean matrix, S overlap matrix.
-  ! ... On output both matrix are unchanged
-  !
-  ! ... Parallel version, with full data distribution
+  !! Called by pdiaghg interface.
+  !! Calculates eigenvalues and eigenvectors of the generalized problem.
+  !! Solve Hv = eSv, with H symmetric matrix, S overlap matrix.
+  !! complex matrices version.
+  !! On output both matrix are unchanged.
+  !!
+  !! Parallel version with full data distribution
+  !!
   !
   USE laxlib_parallel_include
   USE laxlib_descriptor,      ONLY : la_descriptor, laxlib_intarray_to_desc
   USE laxlib_processors_grid, ONLY : ortho_parent_comm
 #if defined __SCALAPACK
-  USE laxlib_processors_grid, ONLY : ortho_cntx, me_blacs, np_ortho, me_ortho, ortho_comm
+  USE laxlib_processors_grid, ONLY : ortho_cntx, np_ortho, me_ortho, ortho_comm
   USE zhpev_module,     ONLY : pzheevd_drv
 #endif
   !
   IMPLICIT NONE
   !
-  INCLUDE 'laxlib_kinds.fh'
+  include 'laxlib_kinds.fh'
   include 'laxlib_param.fh'
   include 'laxlib_mid.fh'
   include 'laxlib_low.fh'
   !
-  INTEGER, INTENT(IN) :: n, ldh
-    ! dimension of the matrix to be diagonalized
-    ! leading dimension of h, as declared in the calling pgm unit
-  COMPLEX(DP), INTENT(INOUT) :: h(ldh,ldh), s(ldh,ldh)
-    ! actually intent(in) but compilers don't know and complain
-    ! matrix to be diagonalized
-    ! overlap matrix
+  INTEGER, INTENT(IN) :: n
+  !! dimension of the matrix to be diagonalized and number of eigenstates to be calculated
+  INTEGER, INTENT(IN) :: ldh
+  !! leading dimension of h, as declared in the calling pgm unit
+  COMPLEX(DP), INTENT(INOUT) :: h(ldh,ldh)
+  !! matrix to be diagonalized
+  COMPLEX(DP), INTENT(INOUT) :: s(ldh,ldh)
+  !! overlap matrix
   REAL(DP), INTENT(OUT) :: e(n)
-    ! eigenvalues
+  !! eigenvalues
   COMPLEX(DP), INTENT(OUT) :: v(ldh,ldh)
-    ! eigenvectors (column-wise)
+  !! eigenvectors (column-wise)
   INTEGER, INTENT(IN) :: idesc(LAX_DESC_SIZE)
-  !
+  !! laxlib descriptor
+  !  
   TYPE(la_descriptor) :: desc
   !
   INTEGER, PARAMETER  :: root = 0
