@@ -14,7 +14,7 @@ END SUBROUTINE laxlib_end
 
 
 SUBROUTINE laxlib_getval_ ( nproc_ortho, leg_ortho, np_ortho, me_ortho, ortho_comm, ortho_row_comm, ortho_col_comm, &
-  ortho_comm_id, ortho_parent_comm, me_blacs, np_blacs, ortho_cntx, world_cntx, do_distr_diag_inside_bgrp  )
+  ortho_comm_id, ortho_parent_comm, ortho_cntx, do_distr_diag_inside_bgrp  )
   use laxlib_processors_grid, ONLY : &
     nproc_ortho_ => nproc_ortho, &
     leg_ortho_   => leg_ortho, &
@@ -25,10 +25,7 @@ SUBROUTINE laxlib_getval_ ( nproc_ortho, leg_ortho, np_ortho, me_ortho, ortho_co
     ortho_col_comm_ => ortho_col_comm, & 
     ortho_comm_id_  => ortho_comm_id, &
     ortho_parent_comm_ => ortho_parent_comm, &
-    me_blacs_    => me_blacs,  &
-    np_blacs_    => np_blacs, &
     ortho_cntx_  => ortho_cntx, &
-    world_cntx_  => world_cntx, &
     do_distr_diag_inside_bgrp_ => do_distr_diag_inside_bgrp
   IMPLICIT NONE
   INTEGER, OPTIONAL, INTENT(OUT) :: nproc_ortho
@@ -40,10 +37,7 @@ SUBROUTINE laxlib_getval_ ( nproc_ortho, leg_ortho, np_ortho, me_ortho, ortho_co
   INTEGER, OPTIONAL, INTENT(OUT) :: ortho_col_comm
   INTEGER, OPTIONAL, INTENT(OUT) :: ortho_comm_id
   INTEGER, OPTIONAL, INTENT(OUT) :: ortho_parent_comm
-  INTEGER, OPTIONAL, INTENT(OUT) :: me_blacs
-  INTEGER, OPTIONAL, INTENT(OUT) :: np_blacs
   INTEGER, OPTIONAL, INTENT(OUT) :: ortho_cntx
-  INTEGER, OPTIONAL, INTENT(OUT) :: world_cntx
   LOGICAL, OPTIONAL, INTENT(OUT) :: do_distr_diag_inside_bgrp
   IF( PRESENT(nproc_ortho) ) nproc_ortho = nproc_ortho_
   IF( PRESENT(leg_ortho) ) leg_ortho = leg_ortho_
@@ -54,10 +48,7 @@ SUBROUTINE laxlib_getval_ ( nproc_ortho, leg_ortho, np_ortho, me_ortho, ortho_co
   IF( PRESENT(ortho_col_comm) ) ortho_col_comm = ortho_col_comm_
   IF( PRESENT(ortho_comm_id) ) ortho_comm_id = ortho_comm_id_
   IF( PRESENT(ortho_parent_comm) ) ortho_parent_comm = ortho_parent_comm_
-  IF( PRESENT(me_blacs) ) me_blacs = me_blacs_
-  IF( PRESENT(np_blacs) ) np_blacs = np_blacs_
   IF( PRESENT(ortho_cntx) ) ortho_cntx = ortho_cntx_
-  IF( PRESENT(world_cntx) ) world_cntx = world_cntx_
   IF( PRESENT(do_distr_diag_inside_bgrp) ) do_distr_diag_inside_bgrp = do_distr_diag_inside_bgrp_
 END SUBROUTINE
 !
@@ -72,10 +63,7 @@ SUBROUTINE laxlib_get_status_x ( lax_status )
     ortho_col_comm_ => ortho_col_comm, & 
     ortho_comm_id_  => ortho_comm_id, &
     ortho_parent_comm_ => ortho_parent_comm, &
-    me_blacs_    => me_blacs,  &
-    np_blacs_    => np_blacs, &
     ortho_cntx_  => ortho_cntx, &
-    world_cntx_  => world_cntx, &
     do_distr_diag_inside_bgrp_ => do_distr_diag_inside_bgrp
   IMPLICIT NONE
   include 'laxlib_param.fh'
@@ -91,10 +79,7 @@ SUBROUTINE laxlib_get_status_x ( lax_status )
   lax_status(LAX_STATUS_COLCOMM)= ortho_col_comm_
   lax_status(LAX_STATUS_COMMID)= ortho_comm_id_
   lax_status(LAX_STATUS_PARENTCOMM)= ortho_parent_comm_
-  lax_status(LAX_STATUS_MEBLACS)= me_blacs_
-  lax_status(LAX_STATUS_NPBLACS)= np_blacs_
   lax_status(LAX_STATUS_ORTHOCNTX)= ortho_cntx_
-  lax_status(LAX_STATUS_WORLDCNTX)= world_cntx_
   IF( do_distr_diag_inside_bgrp_ ) THEN
      lax_status(LAX_STATUS_DISTDIAG)= 1
   ELSE
@@ -104,7 +89,7 @@ END SUBROUTINE
 
 !----------------------------------------------------------------------------
 
-SUBROUTINE laxlib_start_drv( ndiag_, my_world_comm, parent_comm, do_distr_diag_inside_bgrp_  )
+SUBROUTINE laxlib_start_drv( ndiag_, parent_comm, do_distr_diag_inside_bgrp_  )
     !
     use laxlib_processors_grid
     USE laxlib_parallel_include
@@ -115,47 +100,25 @@ SUBROUTINE laxlib_start_drv( ndiag_, my_world_comm, parent_comm, do_distr_diag_i
     IMPLICIT NONE
     !
     INTEGER, INTENT(INOUT) :: ndiag_  ! (IN) input number of procs in the diag group, (OUT) actual number
-    INTEGER, INTENT(IN) :: my_world_comm ! parallel communicator of the "local" world
     INTEGER, INTENT(IN) :: parent_comm ! parallel communicator inside which the distributed linear algebra group
                                        ! communicators are created
     LOGICAL, INTENT(IN) :: do_distr_diag_inside_bgrp_  ! comme son nom l'indique
     !
-    INTEGER :: mpime      =  0  ! the global MPI task index (used in clocks) can be set with a laxlib_rank call
-    !
     INTEGER :: nproc_ortho_try
     INTEGER :: parent_nproc ! nproc of the parent group
-    INTEGER :: world_nproc  ! nproc of the world group
     INTEGER :: my_parent_id ! id of the parent communicator 
-    INTEGER :: nparent_comm ! mumber of parent communicators
     INTEGER :: ierr = 0
     !
     IF( lax_is_initialized ) &
-       CALL laxlib_end_drv ( ) 
+       CALL laxlib_end_drv ( )
 
-    world_nproc  = laxlib_size( my_world_comm ) ! the global number of processors in world_comm
-    mpime        = laxlib_rank( my_world_comm ) ! set the global MPI task index  (used in clocks)
-    parent_nproc = laxlib_size( parent_comm )! the number of processors in the current parent communicator
-    my_parent_id = mpime / parent_nproc  ! set the index of the current parent communicator
-    nparent_comm = world_nproc/parent_nproc ! number of paren communicators
+    parent_nproc = laxlib_size( parent_comm ) ! the number of processors in the current parent communicator
+    my_parent_id = laxlib_rank( parent_comm ) ! set the index of the current parent communicator
 
     ! save input value inside the module
     do_distr_diag_inside_bgrp = do_distr_diag_inside_bgrp_ 
 
     !
-#if defined __SCALAPACK
-    np_blacs     = laxlib_size( my_world_comm )
-    me_blacs     = laxlib_rank( my_world_comm )
-    !
-    ! define a 1D grid containing all MPI tasks of the global communicator
-    ! NOTE: world_cntx has the MPI communicator on entry and the BLACS context on exit
-    !       BLACS_GRIDINIT() will create a copy of the communicator, which can be
-    !       later retrieved using CALL BLACS_GET(world_cntx, 10, comm_copy)
-    !
-    world_cntx = my_world_comm 
-    CALL BLACS_GRIDINIT( world_cntx, 'Row', 1, np_blacs )
-    !
-#endif
-    !
     IF( ndiag_ > 0 ) THEN
        ! command-line argument -ndiag N or -northo N set to a value N
        ! use the command line value ensuring that it falls in the proper range
@@ -173,7 +136,7 @@ SUBROUTINE laxlib_start_drv( ndiag_, my_world_comm, parent_comm, do_distr_diag_i
     ! the ortho group for parallel linear algebra is a sub-group of the pool,
     ! then there are as many ortho groups as pools.
     !
-    CALL init_ortho_group ( nproc_ortho_try, my_world_comm, parent_comm, nparent_comm, my_parent_id )
+    CALL init_ortho_group ( nproc_ortho_try, parent_comm )
     !
     ! set the number of processors in the diag group to the actual number used
     !
@@ -185,14 +148,11 @@ SUBROUTINE laxlib_start_drv( ndiag_, my_world_comm, parent_comm, do_distr_diag_i
     !
 CONTAINS
 
-  SUBROUTINE init_ortho_group ( nproc_try_in, my_world_comm, comm_all, nparent_comm, my_parent_id )
+  SUBROUTINE init_ortho_group ( nproc_try_in, comm_all )
     !
     IMPLICIT NONE
 
     INTEGER, INTENT(IN) :: nproc_try_in, comm_all
-    INTEGER, INTENT(IN) :: my_world_comm ! parallel communicator of the "local" world
-    INTEGER, INTENT(IN) :: nparent_comm
-    INTEGER, INTENT(IN) :: my_parent_id ! id of the parent communicator 
 
     INTEGER :: ierr, color, key, me_all, nproc_all, nproc_try
 
@@ -206,7 +166,6 @@ CONTAINS
 #if defined __MPI
 
     me_all    = laxlib_rank( comm_all )
-    !
     nproc_all = laxlib_size( comm_all )
     !
     nproc_try = MIN( nproc_try_in, nproc_all )
@@ -282,68 +241,21 @@ CONTAINS
        CALL laxlib_comm_split( ortho_comm, me_ortho(2), me_ortho(1), ortho_col_comm)
        CALL laxlib_comm_split( ortho_comm, me_ortho(1), me_ortho(2), ortho_row_comm)
 
+#if defined __SCALAPACK
+       !
+       ortho_cntx = ortho_comm
+       ! ortho_cntx is both an input and output. input is a system context. output is a BLACS context.
+       ! In Fortran, a system context is just a MPI communicator. To avoid any unintended behavior,
+       ! make sure ortho_comm matches exactly the BLACS processor grid.
+       call BLACS_GRIDINIT(ortho_cntx, 'R', np_ortho(1), np_ortho(2))
+#endif
+
     else
        ortho_comm_id = 0
        me_ortho(1) = me_ortho1
        me_ortho(2) = me_ortho1
     endif
 
-#if defined __SCALAPACK
-    !
-    !  This part is used to eliminate the image dependency from ortho groups
-    !  SCALAPACK is now independent from whatever level of parallelization
-    !  is present on top of pool parallelization
-    !
-    ALLOCATE( ortho_cntx_pe( nparent_comm ) )
-    ALLOCATE( blacsmap( np_ortho(1), np_ortho(2) ) )
-
-    DO j = 1, nparent_comm
-
-         CALL BLACS_GET(world_cntx, 10, ortho_cntx_pe( j ) ) ! retrieve communicator of world context
-         blacsmap = 0
-         nprow = np_ortho(1)
-         npcol = np_ortho(2)
-
-         IF( ( j == ( my_parent_id + 1 ) ) .and. ( ortho_comm_id > 0 ) ) THEN
-
-           blacsmap( me_ortho(1) + 1, me_ortho(2) + 1 ) = BLACS_PNUM( world_cntx, 0, me_blacs )
-
-         END IF
-
-         ! All MPI tasks defined in the global communicator take part in the definition of the BLACS grid
-
-         CALL MPI_ALLREDUCE( MPI_IN_PLACE, blacsmap,  SIZE(blacsmap), MPI_INTEGER, MPI_SUM, my_world_comm, ierr )
-         IF( ierr /= 0 ) &
-            CALL lax_error__( ' init_ortho_group ', ' problem in MPI_ALLREDUCE of blacsmap ', ierr )
-
-         CALL BLACS_GRIDMAP( ortho_cntx_pe( j ), blacsmap, nprow, nprow, npcol )
-
-         CALL BLACS_GRIDINFO( ortho_cntx_pe( j ), nprow, npcol, myrow, mycol )
-
-         IF( ( j == ( my_parent_id + 1 ) ) .and. ( ortho_comm_id > 0 ) ) THEN
-
-            IF(  np_ortho(1) /= nprow ) &
-               CALL lax_error__( ' init_ortho_group ', ' problem with SCALAPACK, wrong no. of task rows ', 1 )
-            IF(  np_ortho(2) /= npcol ) &
-               CALL lax_error__( ' init_ortho_group ', ' problem with SCALAPACK, wrong no. of task columns ', 1 )
-            IF(  me_ortho(1) /= myrow ) &
-               CALL lax_error__( ' init_ortho_group ', ' problem with SCALAPACK, wrong task row ID ', 1 )
-            IF(  me_ortho(2) /= mycol ) &
-               CALL lax_error__( ' init_ortho_group ', ' problem with SCALAPACK, wrong task columns ID ', 1 )
-
-            ortho_cntx = ortho_cntx_pe( j )
-
-         END IF
-
-    END DO 
-
-    DEALLOCATE( blacsmap )
-    DEALLOCATE( ortho_cntx_pe )
-
-    !  end SCALAPACK code block
-
-#endif
-
 #else
 
     ortho_comm_id = 1
