@@ -8,30 +8,41 @@
 !
 !----------------------------------------------------------------------------
 SUBROUTINE laxlib_rdiaghg( n, m, h, s, ldh, e, v, me_bgrp, root_bgrp, intra_bgrp_comm )
-  !----------------------------------------------------------------------------
-  ! ... Hv=eSv, with H symmetric matrix, S overlap matrix.
-  ! ... On output both matrix are unchanged
-  !
-  ! ... LAPACK version - uses both DSYGV and DSYGVX
+  !!----------------------------------------------------------------------------
+  !!
+  !! Called by diaghg interface.
+  !! Calculates eigenvalues and eigenvectors of the generalized problem.
+  !! Solve Hv = eSv, with H symmetric matrix, S overlap matrix.
+  !! real matrices version.
+  !! On output both matrix are unchanged.
+  !!
+  !! LAPACK version - uses both DSYGV and DSYGVX
   !
   USE laxlib_parallel_include
   !
   IMPLICIT NONE
-  INCLUDE 'laxlib_kinds.fh'
-  !
-  INTEGER, INTENT(IN) :: n, m, ldh
-    ! dimension of the matrix to be diagonalized
-    ! number of eigenstates to be calculated
-    ! leading dimension of h, as declared in the calling pgm unit
-  REAL(DP), INTENT(INOUT) :: h(ldh,n), s(ldh,n)
-    ! matrix to be diagonalized
-    ! overlap matrix
+  include 'laxlib_kinds.fh'
   !
+  INTEGER, INTENT(IN) :: n
+  !! dimension of the matrix to be diagonalized
+  INTEGER, INTENT(IN) :: m
+  !! number of eigenstates to be calculated
+  INTEGER, INTENT(IN) :: ldh
+  !! leading dimension of h, as declared in the calling pgm unit
+  REAL(DP), INTENT(INOUT) :: h(ldh,n)
+  !! matrix to be diagonalized
+  REAL(DP), INTENT(INOUT) :: s(ldh,n)
+  !! overlap matrix
   REAL(DP), INTENT(OUT) :: e(n)
-    ! eigenvalues
+  !! eigenvalues
   REAL(DP), INTENT(OUT) :: v(ldh,m)
-    ! eigenvectors (column-wise)
-  INTEGER,  INTENT(IN)  :: me_bgrp, root_bgrp, intra_bgrp_comm
+  !! eigenvectors (column-wise)
+  INTEGER,  INTENT(IN)  :: me_bgrp
+  !! index of the processor within a band group
+  INTEGER,  INTENT(IN)  :: root_bgrp
+  !! index of the root processor within a band group
+  INTEGER,  INTENT(IN)  :: intra_bgrp_comm
+  !! intra band group communicator
   !
   INTEGER               :: lwork, nb, mm, info, i, j
     ! mm = number of calculated eigenvectors
@@ -177,8 +188,14 @@ END SUBROUTINE laxlib_rdiaghg
 !----------------------------------------------------------------------------
 SUBROUTINE laxlib_rdiaghg_gpu( n, m, h_d, s_d, ldh, e_d, v_d, me_bgrp, root_bgrp, intra_bgrp_comm )
   !----------------------------------------------------------------------------
-  ! ... Hv=eSv, with H symmetric matrix, S overlap matrix.
-  ! ... On output both matrix are unchanged
+  !!
+  !! Called by diaghg interface.
+  !! Calculates eigenvalues and eigenvectors of the generalized problem
+  !! Solve Hv = eSv, with H symmetric matrix, S overlap matrix.
+  !! real matrices version.
+  !! On output both matrix are unchanged.
+  !!
+  !! GPU VERSION.
   !
 #if defined(_OPENMP)
   USE omp_lib
@@ -205,24 +222,31 @@ SUBROUTINE laxlib_rdiaghg_gpu( n, m, h_d, s_d, ldh, e_d, v_d, me_bgrp, root_bgrp
 #endif
   !
   IMPLICIT NONE
-  INCLUDE 'laxlib_kinds.fh'
-  !
-  INTEGER, INTENT(IN) :: n, m, ldh
-    ! dimension of the matrix to be diagonalized
-    ! number of eigenstates to be calculated
-    ! leading dimension of h, as declared in the calling pgm unit
-  REAL(DP), INTENT(INOUT) :: h_d(ldh,n), s_d(ldh,n)
-    ! matrix to be diagonalized, allocated on the device
-    ! overlap matrix, allocated on the device
+  include 'laxlib_kinds.fh'
   !
+  INTEGER, INTENT(IN) :: n
+  !! dimension of the matrix to be diagonalized
+  INTEGER, INTENT(IN) :: m
+  !! number of eigenstates to be calculated
+  INTEGER, INTENT(IN) :: ldh
+  !! leading dimension of h, as declared in the calling pgm unit
+  REAL(DP), INTENT(INOUT) :: h_d(ldh,n)
+  !! matrix to be diagonalized, allocated on the device
+  REAL(DP), INTENT(INOUT) :: s_d(ldh,n)
+  !! overlap matrix, allocated on the device
   REAL(DP), INTENT(OUT) :: e_d(n)
-    ! eigenvalues, allocated on the device
+  !! eigenvalues, allocated on the device
   REAL(DP), INTENT(OUT) :: v_d(ldh, n)
-    ! eigenvectors (column-wise), allocated on the device
+  !! eigenvectors (column-wise), allocated on the device
+  INTEGER,  INTENT(IN)  :: me_bgrp
+  !! index of the processor within a band group
+  INTEGER,  INTENT(IN)  :: root_bgrp
+  !! index of the root processor within a band group
+  INTEGER,  INTENT(IN)  :: intra_bgrp_comm
+  !! intra band group communicator
 #if defined(__CUDA)
     ATTRIBUTES(DEVICE) :: h_d, s_d, e_d, v_d
 #endif
-  INTEGER,  INTENT(IN)  :: me_bgrp, root_bgrp, intra_bgrp_comm
   !
   INTEGER               :: lwork, nb, mm, info, i, j
     ! mm = number of calculated eigenvectors
@@ -438,40 +462,44 @@ END SUBROUTINE laxlib_rdiaghg_gpu
 SUBROUTINE laxlib_prdiaghg( n, h, s, ldh, e, v, idesc )
   !----------------------------------------------------------------------------
   !
-  ! ... calculates eigenvalues and eigenvectors of the generalized problem
-  ! ... Hv=eSv, with H symmetric matrix, S overlap matrix.
-  ! ... On output both matrix are unchanged
-  !
-  ! ... Parallel version with full data distribution
+  !! Called by pdiaghg interface.
+  !! Calculates eigenvalues and eigenvectors of the generalized problem.
+  !! Solve Hv = eSv, with H symmetric matrix, S overlap matrix.
+  !! real matrices version.
+  !! On output both matrix are unchanged.
+  !!
+  !! Parallel version with full data distribution
+  !!
   !
   USE laxlib_parallel_include
   USE laxlib_descriptor, ONLY : la_descriptor, laxlib_intarray_to_desc
   USE laxlib_processors_grid, ONLY : ortho_parent_comm
 #if defined __SCALAPACK
-  USE laxlib_processors_grid, ONLY : ortho_cntx, me_blacs, np_ortho, me_ortho, ortho_comm
+  USE laxlib_processors_grid, ONLY : ortho_cntx, np_ortho, me_ortho, ortho_comm
   USE dspev_module,      ONLY : pdsyevd_drv
 #endif
   !
   IMPLICIT NONE
   !
-  INCLUDE 'laxlib_kinds.fh'
+  include 'laxlib_kinds.fh'
   include 'laxlib_param.fh'
   include 'laxlib_low.fh'
   include 'laxlib_mid.fh'
   !
-  INTEGER, INTENT(IN) :: n, ldh
-    ! dimension of the matrix to be diagonalized and number of eigenstates to be calculated
-    ! leading dimension of h, as declared in the calling pgm unit
-  REAL(DP), INTENT(INOUT) :: h(ldh,ldh), s(ldh,ldh)
-    ! matrix to be diagonalized
-    ! overlap matrix
-  !
+  INTEGER, INTENT(IN) :: n
+  !! dimension of the matrix to be diagonalized and number of eigenstates to be calculated
+  INTEGER, INTENT(IN) :: ldh
+  !! leading dimension of h, as declared in the calling pgm unit
+  REAL(DP), INTENT(INOUT) :: h(ldh,ldh)
+  !! matrix to be diagonalized
+  REAL(DP), INTENT(INOUT) :: s(ldh,ldh)
+  !! overlap matrix
   REAL(DP), INTENT(OUT) :: e(n)
-    ! eigenvalues
+  !! eigenvalues
   REAL(DP), INTENT(OUT) :: v(ldh,ldh)
-    ! eigenvectors (column-wise)
+  !! eigenvectors (column-wise)
   INTEGER, INTENT(IN) :: idesc(LAX_DESC_SIZE)
-  !
+  !! laxlib descriptor 
   INTEGER, PARAMETER    :: root = 0
   INTEGER               :: nx, info
     ! local block size
