@@ -746,8 +746,18 @@ CONTAINS
       lsda  =   magnetization_obj%lsda
       noncolin = magnetization_obj%noncolin  
       lspinorb = magnetization_obj%spinorbit 
-      domag =   magnetization_obj%do_magnetization 
-      tot_magnetization = magnetization_obj%total
+      IF (magnetization_obj%do_magnetization_ispresent) THEN 
+        domag =   magnetization_obj%do_magnetization
+      ELSE 
+        domag = .FALSE.
+      END IF
+      IF (magnetization_obj%total_ispresent) THEN 
+        tot_magnetization = magnetization_obj%total
+      ELSE IF (magnetization_obj%total_vec_ispresent) THEN 
+        tot_magnetization = SQRT(dot_product(magnetization_obj%total_vec, magnetization_obj%total_vec))
+      ELSE 
+        tot_magnetization = 0._DP 
+      END IF 
       !
     END SUBROUTINE qexsd_copy_magnetization
     !-----------------------------------------------------------------------
@@ -771,7 +781,11 @@ CONTAINS
        !
        occupations = TRIM ( band_struct_obj%occupations_kind%occupations ) 
        smearing    = TRIM ( band_struct_obj%smearing%smearing ) 
-       degauss     = band_struct_obj%smearing%degauss
+       IF (band_struct_obj%smearing%degauss_ispresent) THEN 
+         degauss     = band_struct_obj%smearing%degauss
+       ELSE 
+         degauss = 0._DP
+       END IF 
        !   
        IF ( band_struct_obj%starting_k_points%monkhorst_pack_ispresent ) THEN 
           nks_start = 0 
