@@ -13,13 +13,5 @@ $noresave{$key} = "$nosave";
 # labels from external_latex_labels array.
 
 
-$key = q/Sec:para/;
-$external_latex_labels{$key} = q|5|; 
-$noresave{$key} = "$nosave";
-
-$key = q/_/;
-$external_latex_labels{$key} = q|<|; 
-$noresave{$key} = "$nosave";
-
 1;
 
