@@ -9,10 +9,9 @@
 !----------------------------------------------------------------------------
 SUBROUTINE phq_readin()
   !----------------------------------------------------------------------------
-  !
-  !    This routine reads the control variables for the program phononq.
-  !    A second routine, read_file, reads the variables saved to file
-  !    by the self-consistent program.
+  !! This routine reads the control variables for the program \(\texttt{phononq}\).
+  !! A second routine, \(\texttt{read_file}\), reads the variables saved to file
+  !! by the self-consistent program.
   !
   !
   USE kinds,         ONLY : DP
@@ -30,7 +29,6 @@ SUBROUTINE phq_readin()
   USE fixed_occ,     ONLY : tfixed_occ
   USE lsda_mod,      ONLY : lsda, nspin
   USE fft_base,      ONLY : dffts
-  USE spin_orb,      ONLY : domag, lspinorb
   USE cellmd,        ONLY : lmovecell
   USE run_info,      ONLY : title
   USE control_ph,    ONLY : maxter, alpha_mix, lgamma_gamma, epsil, &
@@ -49,7 +47,7 @@ SUBROUTINE phq_readin()
   USE disp,          ONLY : nq1, nq2, nq3, x_q, wq, nqs, lgamma_iq
   USE io_files,      ONLY : tmp_dir, prefix, postfix, create_directory, &
                             check_tempdir, xmlpun_schema
-  USE noncollin_module, ONLY : i_cons, noncolin
+  USE noncollin_module, ONLY : domag, i_cons, noncolin, lspinorb
   USE control_flags, ONLY : iverbosity, modenum
   USE io_global,     ONLY : meta_ionode, meta_ionode_id, ionode, ionode_id, &
                             qestdin, stdout
@@ -166,12 +164,12 @@ SUBROUTINE phq_readin()
   ! ldiag        : if .true. force diagonalization of the dyn mat
   ! lqdir        : if .true. each q writes in its own directory
   ! search_sym   : if .true. analyze symmetry if possible
-  ! nk1,nk2,nk3, k1,k2,k3 : 
+  ! nk1,nk2,nk3, k1,k2,k3 :
   !              when specified in input, the phonon run uses a different
   !              k-point mesh from that used for the charge density.
   !
   ! dvscf_star%open : if .true. write in dvscf_star%dir the dvscf_q
-  !                   'for all q' in the star of q with suffix dvscf_star%ext. 
+  !                   'for all q' in the star of q with suffix dvscf_star%ext.
   !                   The dvscf_q' is written in the basis dvscf_star%basis;
   !                   if dvscf_star%pat is .true. also save a pattern file.
   ! dvscf_star%dir, dvscf_star%ext, dvscf_star%basis : see dvscf_star%open
@@ -180,25 +178,25 @@ SUBROUTINE phq_readin()
    !
   ! elph_nbnd_min,
   ! elph_nbnd_max: if (elph_mat=.true.) it dumps the eph matrix element from elph_nbnd_min
-  !                  to elph_nbnd_max 
+  !                  to elph_nbnd_max
   ! el_ph_ngauss,
-  ! el_ph_nsigma, 
+  ! el_ph_nsigma,
   ! el_ph_sigma  :  if (elph_mat=.true.) it defines the kind and the val-ue of the
   !                 smearing to be used in the eph coupling calculation.
   ! qplot, : if true a list of q points is given in input
-  ! q_in_band_form: if true the input list of q points defines paths 
+  ! q_in_band_form: if true the input list of q points defines paths
   ! q2d, : if .true. the q list define a mesh in a square.
   ! low_directory_check : if .true. only the requested representations
   !                       are searched on file
   !
   ! read_dns_bare : If .true. the code tries to read three files in DFPT+U calculations:
-  !                 dnsorth, dnsbare, d2nsbare 
-  ! d2ns_type     : DFPT+U - the 2nd bare derivative of occupation matrices ns 
+  !                 dnsorth, dnsbare, d2nsbare
+  ! d2ns_type     : DFPT+U - the 2nd bare derivative of occupation matrices ns
   !                 (d2ns_bare matrix). Experimental! This is why it is not documented in Doc.
-  !                 d2ns_type='full': matrix calculated with no approximation. 
-  !                 d2ns_type='fmmp': assume a m <=> m' symmetry. 
+  !                 d2ns_type='full': matrix calculated with no approximation.
+  !                 d2ns_type='fmmp': assume a m <=> m' symmetry.
   !                 d2ns_type='diag': if okvan=.true. the matrix is calculated retaining only
-  !                                     for <\beta_J|\phi_I> products where for J==I.   
+  !                                     for <\beta_J|\phi_I> products where for J==I.
   !                 d2ns_type='dmmp': same as 'diag', but also assuming a m <=> m'.
   ! diagonalization : diagonalization method used in the nscf calc
   ! ldvscf_interpolate: if .true., use Fourier interpolation of phonon potential
@@ -405,8 +403,6 @@ SUBROUTINE phq_readin()
   ENDDO
   IF (niter_ph.LT.1.OR.niter_ph.GT.maxter) CALL errore ('phq_readin', &
        ' Wrong niter_ph ', 1)
-  IF (nmix_ph.LT.1.OR.nmix_ph.GT.5) CALL errore ('phq_readin', ' Wrong &
-       &nmix_ph ', 1)
   IF (iverbosity.NE.0.AND.iverbosity.NE.1) CALL errore ('phq_readin', &
        &' Wrong  iverbosity ', 1)
   IF (fildyn.EQ.' ') CALL errore ('phq_readin', ' Wrong fildyn ', 1)
@@ -424,7 +420,7 @@ SUBROUTINE phq_readin()
   CASE( 'simple'  )
      elph=.true.
      elph_mat=.false.
-     elph_simple=.true. 
+     elph_simple=.true.
      elph_epa=.false.
   CASE( 'epa' )
      elph=.true.
@@ -553,7 +549,7 @@ SUBROUTINE phq_readin()
   ELSE
      CALL mp_bcast(xq, meta_ionode_id, world_comm  )
   ENDIF
-  
+
   IF (.NOT.ldisp) THEN
      lgamma = xq (1) .EQ.0.D0.AND.xq (2) .EQ.0.D0.AND.xq (3) .EQ.0.D0
      IF ( (epsil.OR.zue.or.lraman.or.elop) .AND..NOT.lgamma) &
@@ -723,7 +719,7 @@ SUBROUTINE phq_readin()
         ALLOCATE(x_q(3,nqs))
         ALLOCATE(wq(nqs))
         wq(:)=wqaux(:)
-        x_q(:,1:nqs)=xqaux(:,1:nqs)  
+        x_q(:,1:nqs)=xqaux(:,1:nqs)
      ENDIF
      DEALLOCATE(xqaux)
      DEALLOCATE(wqaux)
@@ -733,7 +729,7 @@ SUBROUTINE phq_readin()
                        ( ABS(x_q(2,iq)) .LT. 1.0e-10_dp ) .AND. &
                        ( ABS(x_q(3,iq)) .LT. 1.0e-10_dp )
      ENDDO
-     WRITE(stdout, '(//5x,"Dynamical matrices for q-points given in input")') 
+     WRITE(stdout, '(//5x,"Dynamical matrices for q-points given in input")')
      WRITE(stdout, '(5x,"(",i4,"q-points):")') nqs
      WRITE(stdout, '(5x,"  N         xq(1)         xq(2)         xq(3) " )')
      DO iq = 1, nqs
@@ -741,6 +737,8 @@ SUBROUTINE phq_readin()
      END DO
   ENDIF
   !
+  IF (reduce_io) io_level=0
+  !
   ! DFPT+U: the occupation matrix ns is read via read_file
   !
   CALL read_file ( )
@@ -751,9 +749,9 @@ SUBROUTINE phq_readin()
   ! read from input (this happens if nk1*nk2*nk3 > 0; otherwise reset_grid
   ! returns .false., leaves the current values, read in read_file, unchanged)
   !
-  newgrid = reset_grid (nk1, nk2, nk3, k1, k2, k3) 
+  newgrid = reset_grid (nk1, nk2, nk3, k1, k2, k3)
   if(newgrid.and.elph_mat)then
-    WRITE(stdout, '(//5x,"WARNING: Wannier elph do not use explicit new grid: nk1 nk2 nk3 ignored")') 
+    WRITE(stdout, '(//5x,"WARNING: Wannier elph do not use explicit new grid: nk1 nk2 nk3 ignored")')
     newgrid=.false.
   end if
   !
@@ -765,12 +763,12 @@ SUBROUTINE phq_readin()
      'cannot start from pw.x data file using Gamma-point tricks',1)
 
   IF (lda_plus_u) THEN
-     ! 
+     !
      WRITE(stdout,'(/5x,a)') "Phonon calculation with DFPT+U; please cite"
      WRITE(stdout,'(5x,a)')  "A. Floris et al., Phys. Rev. B 84, 161102(R) (2011)"
      WRITE(stdout,'(5x,a)')  "A. Floris et al., Phys. Rev. B 101, 064305 (2020)"
      WRITE(stdout,'(5x,a)')  "in publications or presentations arising from this work."
-     ! 
+     !
      IF (U_projection.NE."atomic") CALL errore("phq_readin", &
           " The phonon code for this U_projection_type is not implemented",1)
      IF (lda_plus_u_kind.NE.0) CALL errore("phq_readin", &
@@ -793,7 +791,7 @@ SUBROUTINE phq_readin()
 
   IF (ts_vdw) CALL errore('phq_readin',&
      'The phonon code with TS-VdW is not yet available',1)
-  
+
   IF (ldftd3) CALL errore('phq_readin',&
      'The phonon code with Grimme''s DFT-D3 is not yet available',1)
 
@@ -806,10 +804,10 @@ SUBROUTINE phq_readin()
   IF (okpaw.and.(lraman.or.elop)) CALL errore('phq_readin',&
      'The phonon code with paw and raman or elop is not yet available',1)
 
-  IF (magnetic_sym) THEN 
-     
+  IF (magnetic_sym) THEN
+
      WRITE(stdout,'(/5x,a)') "Phonon calculation in the non-collinear magnetic case;"
-     WRITE(stdout,'(5x,a)')  "please cite A. Urru and A. Dal Corso, Phys. Rev. B 100," 
+     WRITE(stdout,'(5x,a)')  "please cite A. Urru and A. Dal Corso, Phys. Rev. B 100,"
      WRITE(stdout,'(5x,a)')  "045115 (2019) for the theoretical background."
 
      IF (okpaw) CALL errore('phq_readin',&
@@ -827,8 +825,6 @@ SUBROUTINE phq_readin()
   IF (lmovecell) CALL errore('phq_readin', &
       'The phonon code is not working after vc-relax',1)
 
-  IF (reduce_io) io_level=1
-
   if(elph_mat.and.fildvscf.eq.' ') call errore('phq_readin',&
        'el-ph with wannier requires fildvscf',1)
 
@@ -837,7 +833,7 @@ SUBROUTINE phq_readin()
 
   IF(elph.and.nimage>1) call errore('phq_readin',&
        'el-ph with images not implemented',1)
-  
+
   IF (elph.OR.fildvscf /= ' ') lqdir=.TRUE.
 
   IF(dvscf_star%open.and.nimage>1) CALL errore('phq_readin',&
@@ -896,6 +892,10 @@ SUBROUTINE phq_readin()
      IF ((nat_todo /= 0) .and. lgamma_gamma) CALL errore( &
         'phq_readin', 'gamma_gamma tricks with nat_todo &
        & not available. Use nogg=.true.', 1)
+     IF (lda_plus_u .AND. lgamma_gamma) THEN
+        WRITE(stdout,'(5x,a)')  "DFPT+U does not support k=gamma and q=gamma tricks: disabling them..."
+        lgamma_gamma=.FALSE.
+     ENDIF
      !
      IF (nimage > 1 .AND. lgamma_gamma) CALL errore( &
         'phq_readin','gamma_gamma tricks with images not implemented',1)
@@ -935,7 +935,7 @@ SUBROUTINE phq_readin()
   !
   ! end of reading, close unit qestdin, remove temporary input file if existing
   ! FIXME: closing input file here breaks alpha2F.x that reads what follows
-  !!! IF (meta_ionode) ios = close_input_file () 
+  !!! IF (meta_ionode) ios = close_input_file ()
 
   IF (epsil.AND.(lgauss .OR. ltetra)) &
         CALL errore ('phq_readin', 'no elec. field with metals', 1)
