@@ -27,13 +27,14 @@ SUBROUTINE orthoUwfc(save_wfcatom)
   USE klist,      ONLY : nks, xk, ngk, igk_k
   USE ldaU,       ONLY : U_projection, wfcU, nwfcU, copy_U_wfc
   USE wvfct,      ONLY : npwx
-  USE uspp,       ONLY : nkb, vkb, using_vkb
+  USE uspp,       ONLY : nkb, vkb
   USE becmod,     ONLY : allocate_bec_type, deallocate_bec_type, &
                          bec_type, becp, calbec
   USE control_flags,    ONLY : gamma_only
   USE noncollin_module, ONLY : noncolin, npol
   USE mp_bands,         ONLY : use_bgrp_in_hpsi
   USE becmod_subs_gpum, ONLY : using_becp_auto
+  USE uspp_init,        ONLY : init_us_2
   IMPLICIT NONE
   !
   LOGICAL, INTENT(IN) :: save_wfcatom
@@ -96,7 +97,6 @@ SUBROUTINE orthoUwfc(save_wfcatom)
        CALL atomic_wfc (ik, wfcatom)
      ENDIF
      npw = ngk (ik)
-     CALL using_vkb(1)
      CALL init_us_2 (npw, igk_k(1,ik), xk (1, ik), vkb)
      CALL calbec (npw, vkb, wfcatom, becp)
      CALL s_psi (npwx, npw, natomwfc, wfcatom, swfcatom)
@@ -152,7 +152,7 @@ SUBROUTINE orthoUwfc2 (ik)
   USE klist,            ONLY : nks, xk, ngk, igk_k
   USE ldaU,             ONLY : U_projection, wfcU, nwfcU, copy_U_wfc
   USE wvfct,            ONLY : npwx
-  USE uspp,             ONLY : nkb, vkb, using_vkb
+  USE uspp,             ONLY : nkb, vkb
   USE becmod,           ONLY : allocate_bec_type, deallocate_bec_type, &
                                bec_type, becp, calbec
   USE control_flags,    ONLY : gamma_only
@@ -206,7 +206,7 @@ SUBROUTINE orthoUwfc2 (ik)
      !
      ! Allocate the array becp = <beta|wfcatom>
      CALL allocate_bec_type (nkb,natomwfc, becp)
-     CALL using_becp_auto(2); CALL using_vkb(0); 
+     CALL using_becp_auto(2)
      CALL calbec (npw, vkb, wfcatom, becp)
      ! Calculate swfcatom = S * phi
      CALL s_psi (npwx, npw, natomwfc, wfcatom, swfcatom)
@@ -250,11 +250,12 @@ SUBROUTINE orthoatwfc (orthogonalize_wfc)
   USE basis,      ONLY : natomwfc, swfcatom
   USE klist,      ONLY : nks, xk, ngk, igk_k
   USE wvfct,      ONLY : npwx
-  USE uspp,       ONLY : nkb, vkb, using_vkb
+  USE uspp,       ONLY : nkb, vkb
   USE becmod,     ONLY : allocate_bec_type, deallocate_bec_type, &
                          bec_type, becp, calbec
   USE control_flags,    ONLY : gamma_only
   USE noncollin_module, ONLY : noncolin, npol
+  USE uspp_init,        ONLY : init_us_2
   IMPLICIT NONE
   !
   LOGICAL, INTENT(in) :: orthogonalize_wfc
@@ -280,7 +281,6 @@ SUBROUTINE orthoatwfc (orthogonalize_wfc)
        CALL atomic_wfc (ik, wfcatom)
      ENDIF
      npw = ngk (ik)
-     CALL using_vkb(1)
      CALL init_us_2 (npw, igk_k(1,ik), xk (1, ik), vkb)
      CALL calbec (npw, vkb, wfcatom, becp) 
      CALL s_psi (npwx, npw, natomwfc, wfcatom, swfcatom)
