@@ -47,6 +47,7 @@ MODULE pw_restart_new
   IMPLICIT NONE
   !
   CHARACTER(LEN=6), EXTERNAL :: int_to_char
+  REAL(DP),ALLOCATABLE       :: local_charges(:), local_mag(:,:) 
   PRIVATE
   PUBLIC :: pw_write_schema, write_collected_wfc
   PUBLIC :: read_xml_file, read_collected_wfc
@@ -65,11 +66,11 @@ MODULE pw_restart_new
       !                 NB: wavefunctions are not written here in any case
       !
       USE control_flags,        ONLY : istep, conv_ions, &
-                                       lscf, gamma_only, &
+                                       lscf, scf_error, n_scf_steps, &
                                        tqr, tq_smoothing, tbeta_smoothing, &
-                                       noinv, smallmem, &
-                                       mbd_vdw,          &
-                                       llondon, lxdm, ts_vdw, scf_error, n_scf_steps
+                                       gamma_only, noinv, smallmem, &
+                                       lforce=> tprnfor, tstress, &
+                                       mbd_vdw, llondon, lxdm, ts_vdw
       USE constants,            ONLY : e2
       USE realus,               ONLY : real_space
       USE uspp,                 ONLY : okvan
@@ -103,17 +104,17 @@ MODULE pw_restart_new
                                        Hubbard_alpha, Hubbard_alpha_back, nsg, &
                                        Hubbard_J0, Hubbard_beta, Hubbard_U_back, &
                                        is_hubbard, is_hubbard_back, backall
-      USE spin_orb,             ONLY : lspinorb, domag
       USE symm_base,            ONLY : nrot, nsym, invsym, s, ft, irt, &
                                        t_rev, sname, time_reversal, no_t_rev,&
                                        spacegroup
-      USE lsda_mod,             ONLY : nspin, isk, lsda, starting_magnetization, magtot, absmag
-      USE noncollin_module,     ONLY : angle1, angle2, i_cons, mcons, bfield, magtot_nc, &
-                                       lambda
+      USE lsda_mod,             ONLY : nspin, isk, lsda, starting_magnetization, magtot, & 
+                                       absmag, local_charges, local_mag
+      USE noncollin_module,     ONLY : angle1, angle2, i_cons, mcons, bfield, &
+                                       magtot_nc, lambda, domag, lspinorb
       USE funct,                ONLY : get_dft_short, get_nonlocc_name, dft_is_nonlocc
       
       USE scf,                  ONLY : rho
-      USE force_mod,            ONLY : lforce, sumfor, force, sigma, lstres
+      USE force_mod,            ONLY : sumfor, force, sigma
       USE extfield,             ONLY : tefield, dipfield, edir, etotefield, &
                                        emaxpos, eopreg, eamp, el_dipole, ion_dipole,&
                                        gate, zgate, relaxz, block, block_1,&
@@ -201,7 +202,7 @@ MODULE pw_restart_new
       LOGICAL,POINTER            :: ts_isol_pt, dftd3_threebody_pt, ts_vdw_isolated_pt, domag_opt  
       INTEGER,POINTER            :: dftd3_version_pt
       TYPE(smearing_type),TARGET :: smear_obj 
-      TYPE(smearing_type),POINTER:: smear_obj_ptr 
+      TYPE(smearing_type),POINTER:: smear_obj_ptr
 
       NULLIFY( degauss_, demet_, efield_corr, potstat_corr, gatefield_corr) 
       NULLIFY( gate_info_ptr, dipol_ptr, bp_obj_ptr, hybrid_obj, vdw_obj, dftU_obj, lumo_energy, ef_point)  
@@ -503,12 +504,19 @@ MODULE pw_restart_new
 ! ... MAGNETIZATION
 !-------------------------------------------------------------------------------
          !
-         IF (noncolin) THEN 
-            domag_ = domag
-            domag_opt=> domag_
-         END IF
-         CALL qexsd_init_magnetization(output_obj%magnetization, lsda, noncolin, lspinorb, &
-              magtot, magtot_nc, absmag, domag_opt )
+         output_obj%magnetization_ispresent = .TRUE.  
+         IF (noncolin) THEN
+           CALL qexsd_init_magnetization(output_obj%magnetization, lsda, noncolin, lspinorb, TOTAL_MAG_NC = magtot_nc,&
+             ABSOLUTE_MAG = absmag, ATM = upf(1:nsp)%psd, ITYP = ityp, DO_MAGNETIZATION = domag, & 
+             SITE_MAG = local_mag, SITE_CHARGES = local_charges )
+         ELSE IF (lsda) THEN 
+           CALL qexsd_init_magnetization(output_obj%magnetization, lsda, noncolin, lspinorb, TOTAL_MAG = magtot, &
+                ABSOLUTE_MAG = absmag, ATM = upf(1:nsp)%psd, ITYP = ityp, SITE_MAG_POL = local_mag, & 
+                SITE_CHARGES = local_charges) 
+         ELSE 
+           CALL qexsd_init_magnetization(output_obj%magnetization, lsda, noncolin, lspinorb, ABSOLUTE_MAG = 0._DP, &
+                ATM = upf(1:nsp)%psd, ITYP = ityp ) 
+         END IF 
          !
 
 !--------------------------------------------------------------------------------------
@@ -638,9 +646,9 @@ MODULE pw_restart_new
 !------------------------------------------------------------------------------------------------
 ! ... STRESS 
 !------------------------------------------------------------------------------------------------
-         IF ( lstres .and. conv_elec ) THEN
+         IF ( tstress .and. conv_elec ) THEN
             output_obj%stress_ispresent=.TRUE.
-            CALL qexsd_init_stress(output_obj%stress, sigma, lstres ) 
+            CALL qexsd_init_stress(output_obj%stress, sigma, tstress ) 
          ELSE 
             output_obj%stress_ispresent=.FALSE.
             output_obj%stress%lwrite=.FALSE.
@@ -1019,8 +1027,7 @@ MODULE pw_restart_new
            lxdm, ts_vdw, mbd_vdw
       USE Coul_cut_2D,     ONLY : do_cutoff_2D
       USE noncollin_module,ONLY : noncolin, npol, angle1, angle2, bfield, &
-           nspin_lsda, nspin_gga, nspin_mag
-      USE spin_orb,        ONLY : domag, lspinorb
+              nspin_lsda, nspin_gga, nspin_mag, domag, lspinorb
       USE lsda_mod,        ONLY : nspin, isk, lsda, starting_magnetization,&
            current_spin
       USE realus,          ONLY : real_space
@@ -1164,8 +1171,8 @@ MODULE pw_restart_new
          CALL qexsd_copy_symmetry ( output_obj%symmetries, &
               spacegroup, nsym, nrot, s, ft, sname, t_rev, invsym, irt, &
               noinv, nosym, no_t_rev, input_obj%symmetry_flags )
-         
-         CALL qexsd_copy_efield ( input_obj%electric_field, &
+         IF (input_obj%electric_field_ispresent) & 
+           CALL qexsd_copy_efield ( input_obj%electric_field, &
               tefield, dipfield, edir, emaxpos, eopreg, eamp, &
               gate, zgate, block, block_1, block_2, block_height, relaxz )
          
@@ -1182,7 +1189,11 @@ MODULE pw_restart_new
       !! symmetry check - FIXME: must be done in a more consistent way 
       !! IF (nat > 0) CALL checkallsym( nat, tau, ityp)
       !! Algorithmic info
-      do_cutoff_2D = (output_obj%boundary_conditions%assume_isolated == "2D")
+      IF (output_obj%boundary_conditions_ispresent) THEN 
+         do_cutoff_2D = (output_obj%boundary_conditions%assume_isolated == "2D")
+      ELSE 
+         do_cutoff_2D = .FALSE.
+      END IF
       CALL qexsd_copy_algorithmic_info ( output_obj%algorithmic_info, &
            real_space, tqr, okvan, okpaw )
       !
