@@ -14,10 +14,11 @@ MODULE qe_drivers_gga
   !----------------------------------------------------------------------
   !! Contains the GGA drivers that calculate the XC energy and potential.
   !
-  USE kind_l,       ONLY: DP
-  USE dft_par_mod,  ONLY: igcx, igcc, rho_threshold_gga, grho_threshold_gga,&
-                          exx_started, exx_fraction, screening_parameter,   &
-                          gau_parameter
+  USE kind_l,              ONLY: DP
+  USE dft_setting_params,  ONLY: igcx, igcc, rho_threshold_gga,     &
+                                 grho_threshold_gga, exx_started,   &
+                                 exx_fraction, screening_parameter, &
+                                 gau_parameter
   !
   IMPLICIT NONE
   !
@@ -32,7 +33,7 @@ CONTAINS
 !
 !-----------------------------------------------------------------------
 SUBROUTINE gcxc( length, rho_in, grho_in, sx_out, sc_out, v1x_out, &
-                                          v2x_out, v1c_out, v2c_out )
+                                         v2x_out, v1c_out, v2c_out )
   !---------------------------------------------------------------------
   !! Gradient corrections for exchange and correlation - Hartree a.u. 
   !! See comments at the beginning of module for implemented cases
@@ -65,7 +66,7 @@ SUBROUTINE gcxc( length, rho_in, grho_in, sx_out, sc_out, v1x_out, &
   !
   ! ... local variables
   !
-  INTEGER :: ir
+  INTEGER :: ir, iflag ! Added iflag for AH series
   REAL(DP) :: rho, grho
   REAL(DP) :: sx, v1x, v2x
   REAL(DP) :: sx_, v1x_, v2x_
@@ -79,19 +80,14 @@ SUBROUTINE gcxc( length, rho_in, grho_in, sx_out, sc_out, v1x_out, &
   ntids = omp_get_num_threads()
 #endif
   !
-  !
 #if defined(_OPENACC)
-  !
-  IF (igcx==43 .OR. igcc==14) CALL xclib_error( 'gcxc', 'BEEF not available with&
-                                               & OpenACC enabled', 1 )
-  !
-!$acc data copyin(rho_in,grho_in), copyout(sx_out,sc_out,v1x_out,v2x_out,v1c_out,v2c_out)
+!$acc data deviceptr( rho_in(length), grho_in(length), sx_out(length), sc_out(length), &
+!$acc&                v1x_out(length), v2x_out(length), v1c_out(length), v2c_out(length) )
 !$acc parallel loop  
-#endif
-#if defined(_OPENMP) && !defined(_OPENACC)
+#else
 !$omp parallel if(ntids==1) default(none) &
 !$omp private( rho, grho, sx, sx_, sxsr, v1x, v1x_, v1xsr, &
-!$omp          v2x, v2x_, v2xsr, sc, v1c, v2c ) &
+!$omp          v2x, v2x_, v2xsr, sc, v1c, v2c, iflag ) &
 !$omp shared( rho_in, grho_in, length, igcx, exx_started, &
 !$omp         grho_threshold_gga, rho_threshold_gga, gau_parameter, &
 !$omp         screening_parameter, exx_fraction, igcc, v1x_out, v2x_out, &
@@ -177,6 +173,49 @@ SUBROUTINE gcxc( length, rho_in, grho_in, sx_out, sc_out, v1x_out, &
           v2x = v2x - exx_fraction * v2xsr
         ENDIF
         !
+     CASE( 34, 35 ) ! ' AH series for GGA cross checks
+        !
+        iflag = 0
+        IF ( igcx== 34 ) THEN ! PBE-AH cross check
+           CALL pbex( rho, grho, 1, sx, v1x, v2x )
+           iflag = 1 ! AHPB for PBE cross check
+        ELSEIF ( igcx== 35 ) THEN ! PBESOL-AH cross check
+           CALL pbex( rho, grho, 3, sx, v1x, v2x )
+           iflag = 2 ! AHPS for PBEsol-based cross check
+        ENDIF
+        !
+        IF ( iflag == 0) STOP ! CALL xclib_error( " gcxc ", " Sorting GGA-AHs failed ", 1)
+        !
+        IF (exx_started) THEN
+          CALL axsr( iflag, rho, grho, sxsr, v1xsr, v2xsr, screening_parameter )
+          sx  = sx  - exx_fraction * sxsr
+          v1x = v1x - exx_fraction * v1xsr
+          v2x = v2x - exx_fraction * v2xsr
+        ENDIF
+        !
+     CASE( 32, 33, 47 ) ! 'AH series for vdW-DFs, JPCM 34, 025902 (2022)
+        !
+        iflag = 0
+        IF ( igcx == 32) THEN ! vdW-DF-ahcx
+           CALL cx13( rho, grho, sx, v1x, v2x )
+           iflag = 3 ! for cx13 - analytical sr hole
+        ELSEIF ( igcx == 33) THEN ! vdW-DF2-ah
+           CALL rPW86( rho, grho, sx, v1x, v2x )
+           iflag = 4 ! for rPW86 - analytical sr hole
+        ELSEIF ( igcx == 47) THEN ! vdW-DF2-ahtr
+           CALL b86b( rho, grho, 3, sx, v1x, v2x ) 
+           iflag = 6 ! for test-reserve - analytical sr hole
+        ENDIF
+        !
+        IF ( iflag == 0) STOP ! CALL xclib_error( " gcxc ", " Sorting vdW-DF-AHs failed ", 1)
+        !
+        IF (exx_started) THEN
+          CALL axsr( iflag, rho, grho, sxsr, v1xsr, v2xsr, screening_parameter )
+          sx  = sx  - exx_fraction * sxsr
+          v1x = v1x - exx_fraction * v1xsr
+          v2x = v2x - exx_fraction * v2xsr
+        ENDIF
+        !
      CASE( 13 ) ! 'rPW86'
         !
         CALL rPW86( rho, grho, sx, v1x, v2x )
@@ -301,12 +340,9 @@ SUBROUTINE gcxc( length, rho_in, grho_in, sx_out, sc_out, v1x_out, &
            v2x = (1.0_DP - exx_fraction) * v2x
         ENDIF
         !
-#if !defined(_OPENACC)
-     CASE( 43 ) ! 'beefx' --- BEEF unavailable with OpenACC-- Will be enabled soon
-        ! last parameter = 0 means do not add LDA (=Slater) exchange
-        ! (espresso) will add it itself
-        CALL beefx(rho, grho, sx, v1x, v2x, 0)
-#endif
+     CASE( 43 ) ! 'BEEX'
+        !
+        CALL beefx( rho, grho, sx, v1x, v2x, 0 )
         !
      CASE( 44 ) ! 'RPBE'
         !
@@ -379,12 +415,10 @@ SUBROUTINE gcxc( length, rho_in, grho_in, sx_out, sc_out, v1x_out, &
            v2c = 0.871_DP * v2c
         ENDIF
         !
-#if !defined(_OPENACC)
-     CASE( 14 ) ! BEEF unavailable with OpenACC-- Will be enabled soon
+     CASE( 14 ) !'BEEC'
         ! last parameter 0 means: do not add lda contributions
         ! espresso will do that itself
-        call beeflocalcorr(rho, grho, sc, v1c, v2c, 0)
-#endif
+        CALL beeflocalcorr( rho, grho, sc, v1c, v2c, 0 )
         !
      CASE DEFAULT
         !
@@ -399,14 +433,12 @@ SUBROUTINE gcxc( length, rho_in, grho_in, sx_out, sc_out, v1x_out, &
      v2x_out(ir) = v2x   ;  v2c_out(ir) = v2c
      !
   ENDDO
-#if defined(_OPENMP) && !defined(_OPENACC)
+#if defined(_OPENACC)
+!$acc end data
+#else
 !$omp end do
 !$omp end parallel
 #endif
-#if defined(_OPENACC)
-!$acc end data
-#endif
-  !
   !
   RETURN
   !
@@ -440,7 +472,7 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
   !
   ! ... local variables
   !
-  INTEGER  :: ir, is, iflag
+  INTEGER :: ir, iflag
   REAL(DP) :: rho_up, rho_dw, grho2_up, grho2_dw
   REAL(DP) :: v1x_up, v1x_dw, v2x_up, v2x_dw
   REAL(DP) :: sx_up, sx_dw, rnull_up, rnull_dw
@@ -459,17 +491,13 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
   ntids = omp_get_num_threads()
 #endif
   !
-  sx_tot = 0.0_DP
+  !sx_tot = 0.0_DP
   !
 #if defined(_OPENACC)
-  !
-  IF (igcx==43) CALL xclib_error( 'gcx_spin', 'BEEF not available with&
-                                  & OpenACC enabled', 1 )
-  !
-!$acc data copyin(rho_in, grho2_in), copyout(sx_tot, v1x_out, v2x_out)
+!$acc data deviceptr( rho_in(length,2), grho2_in(length,2), sx_tot(length), &
+!$acc&                v1x_out(length,2), v2x_out(length,2) )
 !$acc parallel loop
-#endif
-#if defined(_OPENMP) && !defined(_OPENACC)
+#else
 !$omp parallel if(ntids==1) default(none) &
 !$omp private( rho_up, rho_dw, grho2_up, grho2_dw, rnull_up, rnull_dw, &
 !$omp          sx_up, sx_dw, sxsr_up, sxsr_dw, v1xsr_up, v1xsr_dw, &
@@ -480,7 +508,6 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
 !$omp          gau_parameter )
 !$omp do
 #endif
-  !
   DO ir = 1, length  
      !
      rho_up = rho_in(ir,1)
@@ -538,10 +565,11 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
         v2x_up = 2.0_DP * v2x_up
         v2x_dw = 2.0_DP * v2x_dw
         !
-     CASE( 3, 4, 8, 10, 12, 20, 23, 24, 25, 44, 45 )
+     CASE( 3, 4, 8, 10, 12, 20, 23, 24, 25, 34, 35, 44, 45 )
         ! igcx=3:  PBE,  igcx=4:  revised PBE, igcx=8:  PBE0, igcx=10: PBEsol
         ! igcx=12: HSE,  igcx=20: gau-pbe,     igcx=23: obk8, igcx=24: ob86,
-        ! igcx=25: ev93, igcx=44: RPBE,        igcx=45: W31X
+        ! igcx=25: ev93, igcx=34: PBE-AH, igcx=35: PBESOL-AH,
+        ! igcx=44: RPBE,        igcx=45: W31X
         !
         iflag = 1
         IF ( igcx== 4 ) iflag = 2
@@ -551,6 +579,8 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
         IF ( igcx==25 ) iflag = 7
         IF ( igcx==44 ) iflag = 8
         IF ( igcx==45 ) iflag = 9
+        IF ( igcx==34 ) iflag = 1
+        IF ( igcx==35 ) iflag = 3
         !
         rho_up = 2.0_DP * rho_up     ; rho_dw = 2.0_DP * rho_dw
         grho2_up = 4.0_DP * grho2_up ; grho2_dw = 4.0_DP * grho2_dw
@@ -584,6 +614,35 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
            v2x_up = v2x_up - exx_fraction * v2xsr_up * 2.0_DP
            v2x_dw = v2x_dw - exx_fraction * v2xsr_dw * 2.0_DP
            !
+        ELSEIF ( igcx == 34 .AND. exx_started ) THEN
+           !
+           CALL axsr( 1, rho_up, grho2_up, sxsr_up, v1xsr_up, &
+                                          v2xsr_up, screening_parameter )
+           CALL axsr( 1, rho_dw, grho2_dw, sxsr_dw, v1xsr_dw, &
+                                          v2xsr_dw, screening_parameter )
+           !
+           sx_tot(ir) = sx_tot(ir) - exx_fraction*0.5_DP * ( sxsr_up*rnull_up + &
+                                                             sxsr_dw*rnull_dw )
+           v1x_up = v1x_up - exx_fraction * v1xsr_up
+           v1x_dw = v1x_dw - exx_fraction * v1xsr_dw
+           v2x_up = v2x_up - exx_fraction * v2xsr_up * 2.0_DP
+           v2x_dw = v2x_dw - exx_fraction * v2xsr_dw * 2.0_DP
+           !
+        ELSEIF ( igcx == 35 .AND. exx_started ) THEN
+           !
+           CALL axsr( 2, rho_up, grho2_up, sxsr_up, v1xsr_up, &
+                                          v2xsr_up, screening_parameter )
+           CALL axsr( 2, rho_dw, grho2_dw, sxsr_dw, v1xsr_dw, &
+                                          v2xsr_dw, screening_parameter )
+           !
+           sx_tot(ir) = sx_tot(ir) - exx_fraction*0.5_DP * ( sxsr_up*rnull_up + &
+                                                             sxsr_dw*rnull_dw )
+           v1x_up = v1x_up - exx_fraction * v1xsr_up
+           v1x_dw = v1x_dw - exx_fraction * v1xsr_dw
+           v2x_up = v2x_up - exx_fraction * v2xsr_up * 2.0_DP
+           v2x_dw = v2x_dw - exx_fraction * v2xsr_dw * 2.0_DP
+           !
+
         ELSEIF ( igcx == 20 .AND. exx_started ) THEN
            ! gau-pbe
            !CALL pbexgau_lsd( rho, grho2, sxsr, v1xsr, v2xsr, gau_parameter )
@@ -741,8 +800,8 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
         IF ( exx_started ) THEN
            sx_tot(ir) = (1.0_DP - exx_fraction) * sx_tot(ir)
            v1x_up = (1.0_DP - exx_fraction) * v1x_up
-           v1x_up = (1.0_DP - exx_fraction) * v1x_up
-           v2x_dw = (1.0_DP - exx_fraction) * v2x_dw
+           v1x_dw = (1.0_DP - exx_fraction) * v1x_dw
+           v2x_up = (1.0_DP - exx_fraction) * v2x_up
            v2x_dw = (1.0_DP - exx_fraction) * v2x_dw
         ENDIF
         !
@@ -786,6 +845,53 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
            v2x_dw = (1.0_DP - exx_fraction) * v2x_dw
         ENDIF  
         !
+     CASE( 32, 33, 47 ) ! ! 'AH series for vdW-DFs, JPCM 34, 025902 (2022)
+        !
+        rho_up = 2.0_DP * rho_up     ; rho_dw = 2.0_DP * rho_dw
+        grho2_up = 4.0_DP * grho2_up ; grho2_dw = 4.0_DP * grho2_dw
+        !
+        ! igcx=32:  vdw-df-ahcx
+        ! igcx=33:  vdw-df2-AH
+        ! igcx=47:  vdw-df2-ahtr
+        !
+        iflag = 0
+        IF ( igcx == 32) THEN ! vdW-DF-ahcx
+           CALL cx13( rho_up, grho2_up, sx_up, v1x_up, v2x_up )
+           CALL cx13( rho_dw, grho2_dw, sx_dw, v1x_dw, v2x_dw )
+           iflag = 3 ! for cx13 - sr hole
+        ELSEIF ( igcx == 33) THEN ! vdW-DF2-ah
+           CALL rPW86( rho_up, grho2_up, sx_up, v1x_up, v2x_up )
+           CALL rPW86( rho_dw, grho2_dw, sx_dw, v1x_dw, v2x_dw )
+           iflag = 4 ! for rPW86 - sr hole
+        ELSEIF ( igcx == 47) THEN ! vdW-DF2-ahtr
+           CALL b86b( rho_up, grho2_up, 3, sx_up, v1x_up, v2x_up ) 
+           CALL b86b( rho_dw, grho2_dw, 3, sx_dw, v1x_dw, v2x_dw ) 
+           iflag = 6 ! for test-reserve - sr hole
+        ENDIF
+        !
+        IF ( iflag == 0) THEN
+           STOP ! CALL xclib_error( " gcx_spin ", " Sorting vdW-DF-AHs failed ", 1)
+        ELSE
+          sx_tot(ir) = 0.5_DP * ( sx_up*rnull_up + sx_dw*rnull_dw )
+          v2x_up = 2.0_DP * v2x_up
+          v2x_dw = 2.0_DP * v2x_dw
+        ENDIF
+        !
+        IF ( exx_started ) THEN
+           !
+           CALL axsr( iflag, rho_up, grho2_up, sxsr_up, v1xsr_up, &
+                                          v2xsr_up, screening_parameter )
+           CALL axsr( iflag, rho_dw, grho2_dw, sxsr_dw, v1xsr_dw, &
+                                          v2xsr_dw, screening_parameter )
+           !
+           sx_tot(ir) = sx_tot(ir) - exx_fraction*0.5_DP * ( sxsr_up*rnull_up + &
+                                                             sxsr_dw*rnull_dw )
+           v1x_up = v1x_up - exx_fraction * v1xsr_up
+           v1x_dw = v1x_dw - exx_fraction * v1xsr_dw
+           v2x_up = v2x_up - exx_fraction * v2xsr_up * 2.0_DP
+           v2x_dw = v2x_dw - exx_fraction * v2xsr_dw * 2.0_DP
+        END IF
+        !
      CASE( 40 )                  ! 'c090 for vdw-df-c090' etc
         !
         rho_up = 2.0_DP * rho_up     ; rho_dw = 2.0_DP * rho_dw
@@ -846,24 +952,22 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
            v2x_dw = (1.0_DP - exx_fraction) * v2x_dw
         ENDIF
         !
-#if !defined(_OPENACC)
-     CASE( 43 ) ! 'beefx'  --- BEEF unavailable with OpenACC-- Will be enabled soon
+        ! case igcx == 5 (HCTH) and 6 (OPTX) not implemented
+        ! case igcx == 7 (meta-GGA) must be treated in a separate call to another
+        ! routine: needs kinetic energy density in addition to rho and grad rho
+        !
+     CASE( 43 )                ! BEEX
         !
         rho_up = 2.0_DP * rho_up     ; rho_dw = 2.0_DP * rho_dw
         grho2_up = 4.0_DP * grho2_up ; grho2_dw = 4.0_DP * grho2_dw
         !
-        CALL beefx(rho_up, grho2_up, sx_up, v1x_up, v2x_up, 0)
-        CALL beefx(rho_dw, grho2_dw, sx_dw, v1x_dw, v2x_dw, 0)
+        CALL beefx( rho_up, grho2_up, sx_up, v1x_up, v2x_up, 0 )
+        CALL beefx( rho_dw, grho2_dw, sx_dw, v1x_dw, v2x_dw, 0 )
         !
         sx_tot(ir) = 0.5_DP * (sx_up*rnull_up + sx_dw*rnull_dw)
         v2x_up = 2.0_DP * v2x_up
         v2x_dw = 2.0_DP * v2x_dw
-#endif
-     !
-     ! case igcx == 5 (HCTH) and 6 (OPTX) not implemented
-     ! case igcx == 7 (meta-GGA) must be treated in a separate call to another
-     ! routine: needs kinetic energy density in addition to rho and grad rho
-     !
+        !
      CASE DEFAULT
         !
         sx_tot(ir) = 0.0_DP
@@ -878,12 +982,11 @@ SUBROUTINE gcx_spin( length, rho_in, grho2_in, sx_tot, v1x_out, v2x_out )
      v2x_out(ir,2) = v2x_dw * rnull_dw
      !
   ENDDO
-#if defined(_OPENMP) && !defined(_OPENACC)
-!$omp end do
-!$omp end parallel
-#endif
 #if defined(_OPENACC)
 !$acc end data
+#else
+!$omp end do
+!$omp end parallel
 #endif
   !
   !
@@ -933,13 +1036,10 @@ SUBROUTINE gcc_spin( length, rho_in, zeta_io, grho_in, sc_out, v1c_out, v2c_out
 #endif
   !
 #if defined(_OPENACC)
-  IF (igcc==14) CALL xclib_error( 'gcc_spin', 'BEEF not available with&
-                                  & OpenACC enabled', 1 )
-  !
-!$acc data copyin(rho_in, grho_in), copyout(sc_out, v1c_out, v2c_out), copy(zeta_io)
+!$acc data deviceptr( rho_in(length), zeta_io(length), grho_in(length), &
+!$acc&                sc_out(length), v1c_out(length,2), v2c_out(length) )
 !$acc parallel loop
-#endif
-#if defined(_OPENMP) && !defined(_OPENACC)
+#else
 !$omp parallel if(ntids==1) default(none) &
 !$omp private( rho, zeta, grho, sc, v1c_up, v1c_dw, v2c ) &
 !$omp shared( igcc, sc_out, v1c_out, v2c_out, &
@@ -987,11 +1087,9 @@ SUBROUTINE gcc_spin( length, rho_in, zeta_io, grho_in, sc_out, v1c_out, v2c_out
        !
        CALL pbec_spin( rho, zeta, grho, 2, sc, v1c_up, v1c_dw, v2c )
        !
-#if !defined(_OPENACC)
-    CASE( 14 )                !*****BEEF unavailable with OpenACC-- Will be enabled soon
-       !
-       call beeflocalcorrspin(rho, zeta, grho, sc, v1c_up, v1c_dw, v2c, 0)
-#endif
+    CASE( 14 )
+       !  
+       CALL beeflocalcorrspin( rho, zeta, grho, sc, v1c_up, v1c_dw, v2c, 0 )
        !
     CASE DEFAULT
        !
@@ -1008,12 +1106,11 @@ SUBROUTINE gcc_spin( length, rho_in, zeta_io, grho_in, sc_out, v1c_out, v2c_out
     v2c_out(ir) = v2c
     !
   ENDDO
-#if defined(_OPENMP) && !defined(_OPENACC)
-!$omp end do
-!$omp end parallel
-#endif
 #if defined(_OPENACC)
 !$acc end data
+#else
+!$omp end do
+!$omp end parallel
 #endif
   !
   RETURN
@@ -1068,16 +1165,11 @@ SUBROUTINE gcc_spin_more( length, rho_in, grho_in, grho_ud_in, &
   ntids = omp_get_num_threads()
 #endif    
   !
-  sc  = 0.0_DP
-  v1c = 0.0_DP
-  v2c = 0.0_DP
-  v2c_ud = 0.0_DP
-  !
 #if defined(_OPENACC) 
-!$acc data copyin(rho_in, grho_in, grho_ud_in), copyout(sc, v1c, v2c, v2c_ud)
+!$acc data deviceptr( rho_in(length,2), grho_in(length,2), grho_ud_in(length), &
+!$acc&                sc(length), v1c(length,2), v2c(length,2), v2c_ud(length) )
 !$acc parallel loop
-#endif
-#if defined(_OPENMP) && !defined(_OPENACC) 
+#else 
 !$omp parallel if(ntids==1) default(none) &
 !$omp private( rho_up, rho_dw, grho_up, grho_dw, grho_ud ) &
 !$omp shared( length, rho_in, grho_in, grho_ud_in, &
@@ -1134,17 +1226,16 @@ SUBROUTINE gcc_spin_more( length, rho_in, grho_in, grho_ud_in, &
        !
     CASE DEFAULT
        !
-       !CALL xclib_error(" gcc_spin_more "," gradient correction not implemented ",1)  !***acc test
+       !CALL xclib_error(" gcc_spin_more "," gradient correction not implemented ",1)
        !
     END SELECT
     !
   ENDDO
-#if defined(_OPENMP) && !defined(_OPENACC)
-!$omp end do
-!$omp end parallel
-#endif
 #if defined(_OPENACC)
 !$acc end data
+#else
+!$omp end do
+!$omp end parallel
 #endif
   !
   RETURN
