@@ -143,7 +143,7 @@ SUBROUTINE pbex( rho, grho, iflag, sx, v1x, v2x )
   ! exchange energy gradient part
   ! auxiliary variable for energy calculation
   REAL(DP) :: dxunif, dfx, f1, f2, f3, dfx1
-  REAL(DP) :: p, amu, ab, c, dfxdp, dfxds, upbe, uge, s, ak, aa
+  REAL(DP) :: p, amu, ab, c, dfxdp, dfxds, s, ak
   ! numerical coefficients (NB: c2=(3 pi^2)^(1/3) )
   REAL(DP), PARAMETER :: pi=3.14159265358979323846d0
   REAL(DP), PARAMETER :: third=1._DP/3._DP, c1=0.75_DP/pi,        &
@@ -660,6 +660,299 @@ SUBROUTINE pbexsr( rho, grho, sxsr, v1xsr, v2xsr, omega )
 END SUBROUTINE pbexsr
 !
 !
+!
+!-----------------------------------------------------------------------     
+      SUBROUTINE axsr(IXC,RHO,GRHO,sx,V1X,V2X,OMEGA)
+!$acc routine (axsr) seq
+!-----------------------------------------------------------------------     
+!!!! Per Hyldgaard, No warranties. adapted from the pbesrx version above
+!-----------------------------------------------------------------------
+!
+!      INCLUDE 'cnst.inc'
+      !
+      use kind_l, ONLY : DP
+      !
+      IMPLICIT NONE
+
+      INTEGER :: IXC
+      REAL(DP):: RHO, GRHO, V1X, V2X, OMEGA
+      REAL(DP), PARAMETER :: SMALL=1.D-20, SMAL2=1.D-08
+      REAL(DP), PARAMETER :: US=0.161620459673995492D0, &
+              AX=-0.738558766382022406D0, &
+              UM=0.2195149727645171D0,UK=0.8040D0,UL=UM/UK
+      REAL(DP), PARAMETER :: f1 = -1.10783814957303361_DP, alpha = 2.0_DP/3.0_DP
+      REAL(DP):: RS, VX, FX, AA, RR, EX, S2, S, D1X, D2X, SX, DSDN, DSDG
+!     ==--------------------------------------------------------------==
+
+!      CALL XC(RHO,EX,EC,VX,VC)
+      RS = RHO**(1.0_DP/3.0_DP)
+      VX = (4.0_DP/3.0_DP)*f1*alpha*RS
+
+!      AA    = DMAX1(GRHO,SMAL2)
+      AA    = GRHO
+!      RR    = RHO**(-4.0_DP/3.0_DP)
+      RR    = 1.0_DP/(RHO*RS)
+      EX    = AX/RR
+      S2    = AA*RR*RR*US*US
+
+      S = SQRT(S2)
+      IF(S.GT.8.3D0) THEN
+        S = 8.572844D0 - 18.796223D0/S2
+      ENDIF
+      CALL wggax_analy_erfc(RHO,S,IXC,OMEGA,FX,D1X,D2X)
+      sx = EX*FX        ! - EX
+      DSDN = -4.D0/3.D0*S/RHO
+      V1X = VX*FX + (DSDN*D2X+D1X)*EX   ! - VX
+      DSDG = US*RR
+      V2X = EX*1.D0/SQRT(AA)*DSDG*D2X
+
+!     ==--------------------------------------------------------------==
+      RETURN
+      END SUBROUTINE axsr
+!
+!
+!-----------------------------------------------------------------------     
+      SUBROUTINE wggax_analy_erfc(rho,s,nggatyp,omega,Fx_wgga, &
+                                  dfxdn,dfxds)
+!$acc routine (wggax_analy_erfc) seq
+!--------------------------------------------------------------------
+!
+!     Short-ranged wGGA Enhancement Factor (from erfc, analytical with
+!     gradients)
+!     
+!     This codes the HJS analytical-xc-hole idea, JCP 128, 194 105 (2008).
+!
+!     Copyright Per Hyldgaard, GPL, No warranty, 2016-
+!
+!     Inputs:
+!     rho     - electron density
+!     s       - scaled graident of electron density
+!     omega = default, short-range screening parameter
+!     nggatyp = 1 is refitted to PBEx -- via analytical hole
+!     nggatyp = 2 is fitted to PBEsolx - via analytical hole
+!     nggatyp = 3 is fitted to cx13 - via analytical hole
+!     nggatyp = 4 is fitted to rPW86 - via analytical hole
+!     nggatyp = 5 is fitted to PHexplore - via analytical hole
+!     nggatyp = 6 is fitted to test-reserve - via analytical hole
+!     ....
+!     nggatyp = 7 fitted to PBEx -- HJS fit to analytical hole
+!     nggatyp = 8 fitted to PBEsolx -- HJS fit to analytical hole
+!
+!     Returns:
+!     Fx_wgga - Analytic version of short-range gga enhancement factor
+!     dfxdn   - Derivative of Fx_wgga with respect to density rho
+!     dfxds   - Derivative of Fx_wgga with respect to scaled gradient s.
+!
+!--------------------------------------------------------------------
+
+      use kind_l, ONLY : DP
+!      USE constants, ONLY : pi
+      Implicit None
+      
+      REAL(DP), PARAMETER :: pi=3.14159265358979323846d0
+
+      Real(dp) :: rho,s,omega,Fx_wgga,dfxdn,dfxds
+      integer :: nggatyp
+
+      Real(dp) :: Abar,B,C,D,E
+      parameter (Abar = 0.757211D0, B = -0.106364D0, C = -0.118649D0)
+      parameter (D =  0.609650D0, E = -0.0477963D0)
+
+      Real(dp) :: One, Two, Three, Four, Five, Six, Seven, Eight, Nine
+      parameter(One=1.0D0,Two=2.0D0,Three=3.0D0,Four=4.0D0,Five=5.0D0)
+      parameter(Six=6.0D0,Seven=7.0D0,Eight=8.0D0,Nine=9.0D0)
+
+      Real(dp) :: f12, f32, f52, f72, f13, f23, f25, f45, f65, f54, f18, f38, f58
+      parameter(f12=0.5D0,f32=Three*f12,f52=Five*f12,f72=Seven*f12)
+      parameter (f13=One/Three,f23=Two*f13)
+      parameter (f25=Two/Five, f45=Two*f25, f65=Three*f25,f54=Five/Four)
+      parameter(f18=One/Eight,f38=Three*f18,f58=Five*f18)
+      Real(dp) :: pi2, pisqrt
+      parameter( pi2=pi**Two, pisqrt=pi**f12)
+
+      Real(dp) :: s0val,s0sq
+      parameter (s0val=2.0D0,s0sq=s0val*s0val)
+
+      Real(dp) :: coef1,coef2,coef3
+      parameter (coef1=(Four/Nine)*B,coef2=(Four/Nine),coef3=Eight/Nine)
+
+!      Real(dp) ::, dimension(6) :: a2,a3,a4,a5,a6,a7
+!      Real(dp) ::, dimension(6) :: b1,b2,b3,b4
+!      Real(dp) ::, dimension(6) :: b5,b6,b7,b8,b9
+      Real(dp), dimension(8) :: a2,a3,a4,a5,a6,a7
+      Real(dp), dimension(8) :: b1,b2,b3,b4
+      Real(dp), dimension(8) :: b5,b6,b7,b8,b9
+
+!     HJS-type/ pbe-x pbesol-x cx13 rPW86 explore TestReserve / 
+!  Last two columns are parameters from the original HJS fits for PBE/PBesol
+      data a2 /  0.0154999D0,   4.58809D-3,   2.43873D-3,   6.00962D-7, &
+                 0.0139602D0,   4.56198D-3, 0.0159941D0,   0.0047333D0 /
+      data a3 / -0.0361006D0,  -8.57842D-3,  -4.15263D-3,  0.0402647D0, &
+                -0.0363194D0,  -8.70003D-3, 0.0852995D0,   0.0403304D0 /
+      data a4 /  0.0379567D0,   7.29562D-3,   2.58261D-3, -0.0353219D0, &
+                 0.0469970D0,   7.36958D-3, -0.1603680D0,  -0.0574615D0 /
+      data a5 / -0.0186715D0,  -3.20195D-3,   1.23940D-6,  0.0116112D0, &
+                -0.0317508D0,  -3.02436D-3, 0.1526450D0,   0.0435395D0 /
+      data a6 /  1.74264D-3,   6.04936D-4,  -7.58225D-4,  -1.55532D-4, &
+                 8.26494D-3,   3.86773D-4, -0.0971263D0,  -0.0216251D0 /
+      data a7 /  1.90765D-3,   2.16112D-5,   2.76378D-4,   5.03561D-5, &
+                 1.45383D-3,   9.43741D-5, 0.0422061D0,   0.0063721D0 /
+      data b1 / -2.7062566D0, -2.1449453D0, -2.2030319D0, -1.8779594D0, &
+                -3.0623921D0, -2.2089330D0, 5.3331900D0,   8.5205600D0 /
+      data b2 /  3.3316842D0,  2.0901104D0,  2.1759315D0,  1.5198811D0, &
+                 4.3601225D0,  2.1968353D0, -12.4780000D0, -13.9885000D0 /
+      data b3 / -2.3871819D0, -1.1935421D0, -1.2997841D0, -0.5383109D0, &
+                -3.7025379D0, -1.2662249D0, 11.0988000D0,   9.2858300D0 /
+      data b4 /  1.1197810D0,  0.4476392D0,  0.5347267D0,  0.1352399D0, &
+                 2.0707006D0,  0.4689964D0, -5.1101300D0,  -3.2728700D0 /
+      data b5 / -0.3606638D0, -0.1172367D0, -0.1588798D0, -0.0428465D0, &
+                -0.7578009D0, -0.1165714D0, 1.7146800D0,   0.8434990D0 /
+      data b6 /  0.0841990D0,  0.0231625D0,  0.0367329D0,  0.0117903D0, &
+                 0.1666493D0,  0.0207188D0, -0.6103800D0,  -0.2355430D0 /
+      data b7 / -0.0114719D0,  -3.52782D-3,  -7.73178D-3,   3.37908D-3, &
+                -0.0178278D0,  -2.97718D-3, 0.3075550D0,   0.0847074D0 /
+      data b8 /  1.69283D-3,   5.39942D-4,   1.26670D-3,  -4.93453D-5, &
+                 7.58236D-3,   5.98226D-4, -0.0770547D0,  -0.0171561D0 /
+      data b9 /  1.50540D-3,   1.58065D-5,   8.04175D-7,   7.09955D-6, &
+                 7.01937D-5,   4.67972D-6, 0.0334840D0,   0.0050552D0 /
+!     End HJS-type parameters: JPCM 34, 025902 (2022)
+
+      integer :: i
+      Real(dp) :: s2,s3,s4,s5,s6,s7,s8,s9
+      Real(dp) :: hnom,hdenom,dhnomds,dhdenomds
+      Real(dp) :: hs,dhds
+
+      Real(dp) :: lam,eta,zeta,dzetads
+      Real(dp) :: xi,phi,psi
+      Real(dp) :: alpha,dalphadn,dalphads
+      Real(dp) :: beta,dbetadn,dbetads
+
+      Real(dp) :: chi,dchidn,dchids
+      Real(dp) :: chiP1, chiP1p, dchiP1dn, dchiP1ds
+      Real(dp) :: chiP2, chiP2p, dchiP2dn, dchiP2ds
+      Real(dp) :: chiP3, chiP3p, dchiP3dn, dchiP3ds
+
+      Real(dp) :: dampfac,cfbars,dcfbards
+      Real(dp) :: egbars,degbards
+
+      Real(dp) :: kf,ny,ny2,dnydn
+
+      kf    = (Three*pi2*rho) ** f13
+      ny= omega/kf
+      ny2=ny*ny
+      dnydn= -f13*ny/rho
+
+!      if ((nggatyp.ge.1).or.(nggatyp.le.6)) then
+      if ((nggatyp.ge.1).or.(nggatyp.le.8)) then
+         i = nggatyp
+      else
+         ! call xclib_error('wgga_analy_erfc','Yet to be coded Wcx part',1)
+         stop
+      endif
+
+      s2=s*s
+      s3=s2*s
+      s4=s2*s2
+      s5=s2*s3
+      s6=s3*s3
+      s7=s4*s3
+      s8=s4*s4
+      s9=s4*s5
+
+      hnom = a2(i)*s2+a3(i)*s3+a4(i)*s4
+      hnom = hnom +a5(i)*s5+a6(i)*s6+a7(i)*s7
+      dhnomds = Two*a2(i)*s+Three*a3(i)*s2+Four*a4(i)*s3
+      dhnomds = dhnomds +Five*a5(i)*s4+Six*a6(i)*s5+Seven*a7(i)*s6
+      hdenom = One+b1(i)*s+b2(i)*s2+b3(i)*s3+b4(i)*s4+b5(i)*s5
+      hdenom = hdenom +b6(i)*s6+b7(i)*s7+b8(i)*s8+b9(i)*s9
+      dhdenomds = b1(i)+Two*b2(i)*s+Three*b3(i)*s2+Four*b4(i)*s3
+      dhdenomds = dhdenomds +Five*b5(i)*s4+Six*b6(i)*s5
+      dhdenomds = dhdenomds +Seven*b7(i)*s6+Eight*b8(i)*s7
+      dhdenomds = dhdenomds +Nine*b9(i)*s8
+      hs=hnom/hdenom
+      dhds=dhnomds/hdenom-hnom*dhdenomds/hdenom/hdenom
+
+      zeta = s2*hs
+      dzetads = Two*s*hs+s2*dhds
+      lam=zeta+D
+      eta=zeta+Abar
+
+      dampfac = (One+s2/s0sq)
+      cfbars=C-s2/dampfac/27.0D0 - zeta/Two
+      dcfbards=-Two*s/dampfac/dampfac/27.0D0 - dzetads/Two
+
+      egbars=-f25*cfbars*lam-(f45/Three)*B*lam**Two
+      egbars=egbars-f65*Abar*lam**Three
+      egbars=egbars-f45*pisqrt*lam**f72
+      egbars=egbars-Three*f45*(zeta**f12-eta**f12)*lam**f72
+
+      degbards=-f25*(dcfbards*lam+cfbars*dzetads)
+      degbards=degbards-f23*f45*B*dzetads*lam
+      degbards=degbards-Nine*f25*Abar*dzetads*lam**Two
+      degbards=degbards-Seven*f25*pisqrt*dzetads*lam**f52
+      degbards=degbards-Seven*f65*dzetads*(zeta**f12-eta**f12)*lam**f52
+      degbards=degbards-f65*dzetads*(zeta**(-f12)-eta**(-f12))*lam**f72
+
+      phi=(lam+ny2)**(f12)
+      psi=(eta+ny2)**(f12)
+      xi=(zeta+ny2)**(f12)
+
+      alpha=Two*ny*(xi-psi)
+      dalphadn=dnydn*Two*(xi-psi+ny2/xi-ny2/psi)
+      dalphads=dzetads*(ny/xi-ny/psi)
+
+      beta=Two*zeta*log((ny+xi)/(ny+phi))-Two*eta*log((ny+psi)/(ny+phi))
+      dbetadn=Abar/phi
+      dbetadn=dbetadn+zeta/xi-eta/psi
+      dbetadn=Two*dbetadn*dnydn
+      dbetads=Abar/(ny+phi)/phi+Two*log((ny+xi)/(ny+psi))
+      dbetads=dbetads+zeta/(ny+xi)/xi-eta/(ny+psi)/psi
+      dbetads=dbetads*dzetads
+
+      chi=ny/phi
+      dchidn=dnydn*lam/phi**Three
+      dchids=-f12*chi*dzetads/phi/phi
+
+      chiP1=One-chi
+      chiP1p=-One
+      dchiP1dn=chiP1p*dchidn
+      dchiP1ds=chiP1p*dchids
+
+      chiP2=One-f32*chi+f12*chi**Three
+      chiP2p=-f32*(One-chi**Two)
+      dchiP2dn=chiP2p*dchidn
+      dchiP2ds=chiP2p*dchids
+
+      chiP3=One-Five*f38*chi+f54*chi**Three-f38*chi**Five
+      chiP3p=-Five*f38+Three*f54*chi**Two-Five*f38*chi**Four
+      dchiP3dn=chiP3p*dchidn
+      dchiP3ds=chiP3p*dchids
+
+      Fx_wgga=Abar
+      Fx_wgga=Fx_wgga-coef1*chiP1/lam
+      Fx_wgga=Fx_wgga-coef2*cfbars*chiP2/lam**Two
+      Fx_wgga=Fx_wgga-coef3*egbars*chiP3/lam**Three
+      Fx_wgga=Fx_wgga+alpha+beta
+
+      dfxdn=-coef1*dchiP1dn/lam
+      dfxdn=dfxdn-coef2*cfbars*dchiP2dn/lam**Two
+      dfxdn=dfxdn-coef3*egbars*dchiP3dn/lam**Three
+      dfxdn=dfxdn+dalphadn+dbetadn
+
+      dfxds=-coef1*(dchiP1ds/lam-chiP1*dzetads/lam**Two)
+      dfxds=dfxds-coef2*(dcfbards*chiP2+cfbars*dchiP2ds)/lam**Two
+      dfxds=dfxds+coef2*cfbars*chiP2*(Two*dzetads/lam**Three)
+      dfxds=dfxds-coef3*(degbards*chiP3+egbars*dchiP3ds)/lam**Three
+      dfxds=dfxds+coef3*egbars*chiP3*(Three*dzetads/lam**Four)
+      dfxds=dfxds+dalphads+dbetads
+
+!     ==--------------------------------------------------------------==
+      RETURN
+      END SUBROUTINE wggax_analy_erfc
+!
+!-----------------------------------------------------------------------
+!
+!
 !-----------------------------------------------------------------------
 SUBROUTINE rPW86( rho, grho, sx, v1x, v2x )
 !$acc routine (rPW86) seq
@@ -841,6 +1134,7 @@ END SUBROUTINE sogga
 SUBROUTINE pbexgau( rho, grho, sxsr, v1xsr, v2xsr, alpha_gau )
 !$acc routine (pbexgau) seq
   !-----------------------------------------------------------------------
+  !! PBEX gaussian.
   !
   USE kind_l,  ONLY: DP
   !
@@ -890,28 +1184,30 @@ SUBROUTINE pbe_gauscheme( rho, s, alpha_gau, Fx, dFxdr, dFxds )
 !$acc routine (pbe_gauscheme) seq
        !--------------------------------------------------------------------
        !
+       USE kind_l, ONLY: DP
+       !
        IMPLICIT NONE
        !
-       REAL*8 rho,s,alpha_gau,Fx,dFxdr,dFxds
+       REAL(dp) :: rho,s,alpha_gau,Fx,dFxdr,dFxds
        ! input: charge and squared gradient and alpha_gau
        ! output: GGA enhancement factor of gau-PBE
        ! output: d(Fx)/d(s), d(Fx)/d(rho)
        !
-       REAL*8 Kx, Nx
+       REAL(dp) :: Kx, Nx
        ! PBE96 GGA enhancement factor
        ! GGA enhancement factor of Gaussian Function
        !
-       REAL*8 bx, cx, PI, sqrtpial, Prefac, term_PBE, Third, KsF
-       REAL*8 d1sdr, d1Kxds, d1Kxdr, d1bxdr, d1bxds, d1bxdKx, &
+       REAL(dp) :: bx, cx, PI, sqrtpial, Prefac, term_PBE, Third, KsF
+       REAL(dp) :: d1sdr, d1Kxds, d1Kxdr, d1bxdr, d1bxds, d1bxdKx, &
               d1Nxdbx,d1Nxdr, d1Nxds
        !
-       REAL*8 Zero,One,Two,Three,Four,Five,Six,Seven,Eight,Nine,Ten
+       REAL(dp) :: Zero,One,Two,Three,Four,Five,Six,Seven,Eight,Nine,Ten
        !
        SAVE Zero,One,Two,Three,Four,Five,Six,Seven,Eight,Nine,Ten
        DATA Zero,One,Two,Three,Four,Five,Six,Seven,Eight,Nine,Ten &
          / 0D0,1D0,2D0,3D0,4D0,5D0,6D0,7D0,8D0,9D0,10D0 /
        !
-       REAL*8 k , mu
+       REAL(dp) :: k , mu
        DATA k / 0.804d0 / , mu / 0.21951d0 /
        ! parameters of PBE functional
        !
@@ -938,7 +1234,7 @@ SUBROUTINE pbe_gauscheme( rho, s, alpha_gau, Fx, dFxdr, dFxds )
        Nx = bx * Prefac * ( SQRT(PI) * ERF(One/bx) + & 
         (bx - Two*bx*bx*bx)*cx - Two*bx )
        !
-       ! for convergency
+       ! for convergence
        IF (ABS(Nx) < 1.0D-15) THEN
          Nx = Zero
        ELSEIF ((One - ABS(Nx)) < 1.0D-15) THEN
@@ -946,7 +1242,7 @@ SUBROUTINE pbe_gauscheme( rho, s, alpha_gau, Fx, dFxdr, dFxds )
        ELSE
          Nx = Nx
        ENDIF
-       ! for convergency end
+       ! for convergence end
        !
        Fx =  Kx * Nx
        !
@@ -1241,7 +1537,7 @@ SUBROUTINE becke88_spin( rho_up, rho_dw, grho_up, grho_dw, sx_up, sx_dw, v1x_up,
   !
   ! ... local variables
   !
-  INTEGER :: is
+  !INTEGER :: is
   REAL(DP), PARAMETER :: beta = 0.0042_DP, third = 1._DP/3._DP
   REAL(DP) :: rho13, rho43, xs, xs2, sa2b8, shm1, dd, dd2, ee
   !
@@ -1283,12 +1579,10 @@ END SUBROUTINE becke88_spin
 SUBROUTINE wpbe_analy_erfc_approx_grad( rho, s, omega, Fx_wpbe, d1rfx, d1sfx )
 !$acc routine (wpbe_analy_erfc_approx_grad) seq
       !-----------------------------------------------------------------------
-      !
-      !     wPBE Enhancement Factor (erfc approx.,analytical, gradients)
-      !
-      !--------------------------------------------------------------------
+      !! wPBE Enhancement Factor (erfc approx.,analytical, gradients).
       !
       USE kind_l,    ONLY: DP
+      !
       IMPLICIT NONE
       !
       REAL(DP) rho,s,omega,Fx_wpbe,d1sfx,d1rfx
@@ -1322,7 +1616,6 @@ SUBROUTINE wpbe_analy_erfc_approx_grad( rho, s, omega, Fx_wpbe, d1rfx, d1sfx )
       REAL(DP) DHsbw,DHsbw2,DHsbw3,DHsbw4,DHsbw5
       REAL(DP) DHsbw12,DHsbw32,DHsbw52,DHsbw72,DHsbw92
       REAL(DP) d1sHsbw,d1rHsbw
-      REAL(DP) d1sDHsbw,d1rDHsbw
       REAL(DP) HsbwA94,HsbwA9412
       REAL(DP) HsbwA942,HsbwA943,HsbwA945
       REAL(DP) piexperf,expei
@@ -1352,9 +1645,6 @@ SUBROUTINE wpbe_analy_erfc_approx_grad( rho, s, omega, Fx_wpbe, d1rfx, d1sfx )
       !
       REAL(DP) term1,term2,term3,term4,term5
       !
-      REAL(DP) ax,um,uk,ul
-      REAL(DP) gc1,gc2
-      !
       ! REAL(DP) ei
       !
       REAL(DP) Zero,One,Two,Three,Four,Five,Six,Seven,Eight,Nine,Ten
@@ -1882,11 +2172,12 @@ END SUBROUTINE wpbe_analy_erfc_approx_grad
 FUNCTION EXPINT(n, x)
 !$acc routine (expint) seq
 !-----------------------------------------------------------------------
-! Evaluates the exponential integral E_n(x)
+!! Evaluates the exponential integral \(E_n(x)\). 
+!! Inspired by Numerical Recipes.
 ! Parameters: maxit is the maximum allowed number of iterations,
 ! eps is the desired relative error, not smaller than the machine precision,
 ! big is a number near the largest representable floating-point number,
-! Inspired from Numerical Recipes
+
 !
       USE kind_l,   ONLY: DP
       IMPLICIT NONE
