@@ -40,13 +40,12 @@ CONTAINS
     !! ierr=0  : xml schema, ierr=-2: UPF v.2
     !! ierr=-81: error reading PP file
     !
-    iun = xml_openfile ( filename )
+    iun = xml_open_file ( filename )
     IF ( iun == -1 ) CALL upf_error('read_upf', 'cannot open file',1)
-    print *, ' READ_UPF_NEW'
     call xmlr_opentag ( 'qe_pp:pseudo', IERR = ierr )
     if ( ierr == 0 ) then
        v2 =.false.
-    else if ( ierr == -1 ) then
+    else if ( ierr == 1 ) then
        rewind (iun) 
        call xmlr_opentag ( 'UPF', IERR = ierr )
        if ( ierr == 0 ) then
@@ -56,6 +55,7 @@ CONTAINS
        end if
     end if
     if ( ierr /= 0 .and. ierr /= -2 ) then
+       call xml_closefile( )
        ierr = -81
        return
     end if
@@ -231,7 +231,18 @@ CONTAINS
     !
     CALL xmlr_opentag( capitalize_if_v2('pp_mesh') )
     CALL get_attr ( 'mesh', mesh )
-    if ( mesh /= upf%mesh ) call upf_error('read_pp_mesh','mismatch in mesh',mesh)
+    if ( mesh == 0 ) THEN
+#if defined (__debug)
+       call upf_error('read_pp_mesh',&
+         'mesh size missing, using the one in header',-1)
+#else
+       continue
+#endif
+    else if ( mesh /= upf%mesh ) THEN
+       call upf_error('read_pp_mesh',&
+         'mismatch in mesh size, discarding the one in header',-1)
+       upf%mesh = mesh
+    end if
     CALL get_attr ( 'dx'  , upf%dx   )
     CALL get_attr ( 'xmin', upf%xmin )
     CALL get_attr ( 'rmax', upf%rmax )
@@ -340,7 +351,9 @@ CONTAINS
        END IF
        CALL xmlr_readtag( tag, upf%beta(1:upf%mesh,nb) )
        CALL get_attr('index', mb)
-       IF ( nb /= mb ) CALL upf_error('read_pp_nonlocal','mismatch',nb)
+       ! not-so-strict test: index is absent or incorrect in some UPF v.2 files
+       IF ( .NOT. v2 .AND. nb /= mb ) &
+            CALL upf_error('read_pp_nonlocal','mismatch',nb)
        CALL get_attr('label', upf%els_beta(nb))
        CALL get_attr('angular_momentum', upf%lll(nb))
        IF ( .NOT. v2 .AND. upf%has_so ) &
@@ -353,9 +366,7 @@ CONTAINS
     !
     ! pp_dij (D_lm matrix)
     !
-    CALL xmlr_opentag( capitalize_if_v2 ('pp_dij') )
-    READ(iun,*) upf%dion(1:upf%nbeta,1:upf%nbeta)
-    CALL xmlr_closetag( ) 
+    CALL xmlr_readtag ( capitalize_if_v2 ('pp_dij'), upf%dion )
     !
     ! pp_augmentation
     !
@@ -386,37 +397,40 @@ CONTAINS
           ENDIF
        ENDIF
        !
-       CALL xmlr_opentag( capitalize_if_v2('pp_q') )
-       READ(iun,*) upf%qqq(1:upf%nbeta,1:upf%nbeta)
-       CALL xmlr_closetag( )
+       CALL xmlr_readtag( capitalize_if_v2('pp_q'), upf%qqq )
        !
        IF ( upf%tpawp ) THEN
-          CALL xmlr_opentag( capitalize_if_v2('pp_multipoles') )
           ALLOCATE ( upf%paw%augmom(1:upf%nbeta,1:upf%nbeta,0:2*upf%lmax) )
-          READ(iun,*) upf%paw%augmom(1:upf%nbeta,1:upf%nbeta,0:2*upf%lmax)
-          CALL xmlr_closetag ()
+          CALL xmlr_readtag( capitalize_if_v2('pp_multipoles'), upf%paw%augmom )
        ENDIF
        !
        ! read polinomial coefficients for Q_ij expansion at small radius
        !
+       IF ( upf%nqlc == 0 ) upf%nqlc = 2*upf%lmax+1
+       ALLOCATE( upf%rinner( upf%nqlc ) )
        IF ( v2 .AND. upf%nqf > 0) THEN
           ALLOCATE ( upf%qfcoef(upf%nqf, upf%nqlc, upf%nbeta, upf%nbeta) )
           CALL xmlr_opentag('PP_QFCOEF')
           READ(iun,*) upf%qfcoef
           CALL xmlr_closetag ()
-          ALLOCATE( upf%rinner( upf%nqlc ) )
           CALL xmlr_readtag('PP_RINNER',upf%rinner)
        ELSE IF ( upf%nqf == 0 ) THEN
-          ALLOCATE( upf%rinner(1), upf%qfcoef(1,1,1,1) )
-          upf%rinner = 0.0_dp; upf%qfcoef =0.0_dp
+          ALLOCATE( upf%qfcoef(1,1,1,1) )
+          upf%qfcoef =0.0_dp
        ENDIF
        !
        ! Read augmentation charge Q_ij
        !
        IF( upf%q_with_l ) THEN
           ALLOCATE( upf%qfuncl(upf%mesh,upf%nbeta*(upf%nbeta+1)/2,0:2*upf%lmax) )
+          upf%qfuncl(:,:,:) = 0.0_dp
+          ! NOTE: it would be wiser to dimension qfuncl as (:,:,0:upf%lmax)
+          ! and store the q_l(r) with index l=L/2 (see loop_on_l below)
+          ! This would save some storage and avoid "holes" in the array
+          ! that may be a source of trouble if not initialized to zero 
        ELSE
           ALLOCATE ( upf%qfunc(upf%mesh,upf%nbeta*(upf%nbeta+1)/2) )
+          upf%qfunc (:,:) = 0.0_dp
        END IF
        ALLOCATE ( aux(upf%mesh) )
        loop_on_nb: DO nb = 1,upf%nbeta
@@ -506,7 +520,8 @@ CONTAINS
        END IF
        CALL xmlr_readtag( tag, upf%chi(1:upf%mesh,nw) )
        call get_attr('index', ind)
-       if ( ind /= nw ) &
+       ! not-so-strict test: index is absent or incorrect in some UPF v.2 files
+       if ( .NOT. v2 .AND. ind /= nw ) &
             call upf_error('read_pp_pswfc','mismatch reading PSWFC', nw)
        call get_attr( 'label', upf%els(nw) )
        call get_attr( 'l', upf%lchi(nw) )
@@ -547,20 +562,22 @@ CONTAINS
           END IF
           CALL xmlr_readtag( tag, upf%aewfc(1:upf%mesh,nb) )
           CALL get_attr ('index',mb)
-          IF ( nb /= mb ) CALL upf_error('read_pp_full_wfc','mismatch',1)
+          ! not-so-strict test (and two more below):
+          ! index may be absent or incorrect in some UPF v.2 files
+          IF ( .NOT. v2 .AND. nb /= mb ) CALL upf_error('read_pp_full_wfc','mismatch',1)
        END DO
        !
        IF ( upf%has_so .AND. upf%tpawp ) THEN
           ALLOCATE (upf%paw%aewfc_rel(1:upf%mesh,upf%nbeta) )
           DO nb = 1, upf%nbeta
              IF ( v2 ) THEN
-                tag = 'PP_AEWFC_rel.'//i2c(nb)
+                tag = 'PP_AEWFC_REL.'//i2c(nb)
              ELSE
                 tag = 'pp_aewfc_rel'
              END IF
              CALL xmlr_readtag(tag, upf%paw%aewfc_rel(1:upf%mesh,nb) )
              CALL get_attr ('index',mb)
-             IF ( nb /= mb ) CALL upf_error('read_pp_full_wfc','mismatch',2)
+             IF ( .NOT. v2 .AND. nb /= mb ) CALL upf_error('read_pp_full_wfc','mismatch',2)
           END DO
        END IF
        !
@@ -573,7 +590,7 @@ CONTAINS
           END IF
           CALL xmlr_readtag(tag, upf%pswfc(1:upf%mesh,nb) )
           CALL get_attr ('index',mb)
-          IF ( nb /= mb ) CALL upf_error('read_pp_full_wfc','mismatch',3)
+          IF ( .NOT. v2 .AND. nb /= mb ) CALL upf_error('read_pp_full_wfc','mismatch',3)
        END DO
        !
        CALL xmlr_closetag( )
@@ -597,12 +614,18 @@ CONTAINS
     DO nw = 1,upf%nwfc
        CALL xmlr_readtag( 'PP_RELWFC.'//i2c(nw), dummy )
        CALL get_attr( 'index' , nb )
-       IF ( nb /= nw ) CALL upf_error('read_pp_spinorb','mismatch',1)
-       CALL get_attr( 'els',   upf%els(nw) )
+       ! not-so-strict test: index absent or incorrect in some UPF v.2 files
+       IF ( .NOT. v2 .AND. nb /= nw ) CALL upf_error('read_pp_spinorb','mismatch',1)
        CALL get_attr( 'nn',    upf%nn(nw) )
-       CALL get_attr( 'lchi',  upf%lchi(nw) )
        CALL get_attr( 'jchi',  upf%jchi(nw) )
-       CALL get_attr( 'oc',    upf%oc(nw) )
+       !
+       ! the following data is already known and was not read in old versions
+       ! of UPF-reading code. upf%oc is actually missing in some UPF files:
+       ! reading it here may spoil the value read earlier and break DFT+U
+       !
+       ! CALL get_attr( 'lchi',  upf%lchi(nw) )
+       ! CALL get_attr( 'els',   upf%els(nw) )
+       ! CALL get_attr( 'oc',    upf%oc(nw) )
     ENDDO
     !
     DO nb = 1,upf%nbeta
@@ -611,12 +634,12 @@ CONTAINS
        ! existing PP files may have pp_relbeta first, pp_relwfc later,
        ! but also the other way round - check that everything was right
        !
-       if ( ierr /= 0 ) then
+       if ( ierr > 0 ) then
           ierr = -81
           return
        end if
        CALL get_attr( 'index' , nw )
-       IF ( nb /= nw ) CALL upf_error('read_pp_spinorb','mismatch',2)
+       IF ( .NOT.v2 .AND. nb /= nw ) CALL upf_error('read_pp_spinorb','mismatch',2)
        CALL get_attr( 'lll',  upf%lll(nb) )
        CALL get_attr( 'jjj',  upf%jjj(nb) )
     ENDDO
@@ -703,7 +726,7 @@ CONTAINS
     IMPLICIT NONE
     TYPE(pseudo_upf),INTENT(INOUT) :: upf ! the pseudo data
     !
-    INTEGER :: nb, mb
+    INTEGER :: nb, mb, ierr
     CHARACTER(LEN=24) :: tag
     !
     IF (.NOT. upf%has_gipaw) RETURN
@@ -711,10 +734,14 @@ CONTAINS
     CALL xmlr_opentag( capitalize_if_v2('pp_gipaw') )
     CALL get_attr ('gipaw_data_format', upf%gipaw_data_format ) 
     IF ( v2 ) THEN
-       CALL xmlr_opentag( 'PP_GIPAW_CORE_ORBITALS')
+       CALL xmlr_opentag( 'PP_GIPAW_CORE_ORBITALS', IERR=ierr )
+       ! ierr= 0 for case <PP_GIPAW_CORE_ORBITALS>...</PP_GIPAW_CORE_ORBITALS>
+       ! ierr=-1 for case <PP_GIPAW_CORE_ORBITALS ... />
        CALL get_attr ('number_of_core_orbitals', upf%gipaw_ncore_orbitals)
     ELSE
-       print *, 'FIXME! upf%gipaw_ncore_orbitals'
+       CALL xmlr_readtag ('number_of_core_orbitals', upf%gipaw_ncore_orbitals) 
+       IF ( .NOT. upf%paw_as_gipaw) & 
+          CALL xmlr_readtag( 'number_of_valence_orbitals', upf%gipaw_wfs_nchannels)  
     END IF
     ALLOCATE ( upf%gipaw_core_orbital(upf%mesh,upf%gipaw_ncore_orbitals) )
     ALLOCATE ( upf%gipaw_core_orbital_n(upf%gipaw_ncore_orbitals) )
@@ -733,7 +760,8 @@ CONTAINS
        CALL get_attr ('n', upf%gipaw_core_orbital_n(nb) )
        CALL get_attr ('l', upf%gipaw_core_orbital_l(nb) )
     END DO
-    IF ( v2 ) CALL xmlr_closetag ( )
+    ! close only for case <PP_GIPAW_CORE_ORBITALS> ... </PP_GIPAW_CORE_ORBITALS>
+    IF ( v2 .AND. ierr == 0  ) CALL xmlr_closetag ( )
     !
     IF ( upf%paw_as_gipaw) THEN
        !
@@ -771,8 +799,6 @@ CONTAINS
           CALL xmlr_opentag( 'PP_GIPAW_ORBITALS' )
           CALL get_attr( 'number_of_valence_orbitals', &
                upf%gipaw_wfs_nchannels )
-       ELSE
-          print *, 'FIXME! upf%gipaw_wfs_nchannel'
        END IF
        ALLOCATE ( upf%gipaw_wfs_el(upf%gipaw_wfs_nchannels) )
        ALLOCATE ( upf%gipaw_wfs_ll(upf%gipaw_wfs_nchannels) )
@@ -780,8 +806,6 @@ CONTAINS
        ALLOCATE ( upf%gipaw_wfs_rcutus(upf%gipaw_wfs_nchannels) )
        ALLOCATE ( upf%gipaw_wfs_ae(upf%mesh,upf%gipaw_wfs_nchannels) )
        ALLOCATE ( upf%gipaw_wfs_ps(upf%mesh,upf%gipaw_wfs_nchannels) )
-       ALLOCATE ( upf%gipaw_vlocal_ae(upf%mesh) )
-       ALLOCATE ( upf%gipaw_vlocal_ps(upf%mesh) )
        DO nb = 1,upf%gipaw_wfs_nchannels
           IF ( v2 ) THEN
              tag = "PP_GIPAW_ORBITAL."//i2c(nb)
@@ -805,6 +829,8 @@ CONTAINS
        !
        ! Read all-electron and pseudo local potentials
        !
+       ALLOCATE ( upf%gipaw_vlocal_ae(upf%mesh) )
+       ALLOCATE ( upf%gipaw_vlocal_ps(upf%mesh) )
        CALL xmlr_opentag( capitalize_if_v2('pp_gipaw_vlocal') )
        CALL xmlr_readtag( capitalize_if_v2('pp_gipaw_vlocal_ae'), &
             upf%gipaw_vlocal_ae(1:upf%mesh) )
