@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2020 Quantum ESPRESSO group
+! Copyright (C) 2020-2021 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -10,7 +10,7 @@ MODULE xmltools
   !--------------------------------------------------------
   !
   ! Poor-man set of tools for reading and writing xml files
-  ! Similar to iotk but much simpler - Paolo Giannozzi, June 2020 
+  ! Similar to iotk but much simpler - Paolo Giannozzi, June 2020
   ! Limitations: too many to be listed in detail. Main ones:
   ! * works on a single opened file at the time. Exception:
   !   while a file is opened, one can open, R/W, close another file,
@@ -24,6 +24,8 @@ MODULE xmltools
   !   tag is found only above the current position, and nowhere else
   ! * only single values (e.g. no vectors) in attributes
   ! * attributes should not contain commas or strange characters
+  ! * xml comments (<!-- ...  -->) or <![CDATA[ ... ]]> cannot be mixed
+  !   with numerical fields
   !
   USE upf_kinds, ONLY : dp
   IMPLICIT NONE
@@ -58,7 +60,7 @@ MODULE xmltools
   !
   PRIVATE
   ! general subroutines
-  PUBLIC :: xml_openfile, xml_closefile
+  PUBLIC :: xml_open_file, xml_closefile
   ! subroutines for writing
   PUBLIC :: add_attr
   PUBLIC :: xmlw_writetag, xmlw_opentag, xmlw_closetag
@@ -68,6 +70,21 @@ MODULE xmltools
   ! utility functions
   PUBLIC :: xml_protect, i2c, l2c, r2c
   !
+  ! Error codes returned by xmlr_opentag / xml_readtag:
+  !  -1   tag with no value (e.g. <tag attr="val"/>) found (no error)
+  !   0   tag found and read (no error)
+  !   1   tag not found
+  !   2   error parsing file
+  !   3   line too long
+  !   4   too many levels of tags
+  ! 
+  ! Error codes returned by xmlw_opentag / xml_writetag:
+  !   0     tag open and/or written (no error)
+  !   1     cannot write to unit "xmlunit"
+  !   2     tag name too long
+  !   3     wrong number of values for attributes
+  !   4     too many levels of tag
+  ! 
   INTERFACE xmlr_readtag
      MODULE PROCEDURE readtag_c, readtag_r, readtag_l, readtag_i, &
           readtag_iv, readtag_rv, readtag_rm, readtag_rt, &
@@ -245,7 +262,7 @@ CONTAINS
     !
   END SUBROUTINE add_c_attr
   !
-  FUNCTION xml_openfile ( filexml ) RESULT (iun)
+  FUNCTION xml_open_file ( filexml ) RESULT (iun)
     !
     ! returns on output the opened unit number if opened successfully
     ! returns -1 otherwise
@@ -272,7 +289,7 @@ CONTAINS
     print "('file ',a,' opened with unit ',i5)",trim(filexml),iun
 #endif
     !
-  END FUNCTION xml_openfile
+  END FUNCTION xml_open_file
   !
   SUBROUTINE xml_closefile ( )
     !
@@ -294,13 +311,8 @@ CONTAINS
     ! name      required, character: tag name
     ! On output: the tag is left open, ready for addition of data -
     !            the tag must be subsequently closed with close_xml_tag
-    ! If ierr is present, the following value is returned:
-    ! ierr = 0     normal execution
-    ! ierr = 1     cannot write to unit "xmlunit"
-    ! ierr = 2     tag name too long
-    ! ierr = 3     too many tag levels
-    ! ierr =10     wrong number of values for attributes
-    ! If absent, the above error messages are printed.
+    ! If ierr is present, the error code set in write_tag_and_attr is returned
+    ! If ierr is absent,  the above error code is reprinted on output
     !
     CHARACTER(LEN=*), INTENT(IN) :: name
     INTEGER, INTENT(OUT),OPTIONAL :: ierr
@@ -543,6 +555,7 @@ CONTAINS
     !
     CHARACTER(LEN=*), INTENT(IN) :: name
     INTEGER :: ierr
+    ! See list of error codes in the header of this file
     !
     LOGICAL :: have_list, have_vals
     INTEGER :: i, la, lv, n1a,n2a, n1v, n2v
@@ -553,7 +566,7 @@ CONTAINS
     END IF
     !
     IF ( nlevel+1 > maxlevel ) THEN
-       ierr = 3
+       ierr = 4
        RETURN
     END IF
     nlevel = nlevel+1
@@ -572,7 +585,7 @@ CONTAINS
     !
     ! attributes (if present)
     !
-    ierr = 10
+    ierr = 3
     if ( allocated (attrlist) ) then
        WRITE (xmlunit, "(A)", ADVANCE='no', ERR=10) attrlist
        deallocate (attrlist)
@@ -915,10 +928,6 @@ CONTAINS
     character(len=*), intent(in) :: tag
     character(len=*), intent(out):: cval
     integer, intent(out), optional :: ierr
-    ! 0: tag found and read
-    !-1: tag not found
-    ! 1: error parsing file
-    ! 2: error in arguments
     !
     integer ::  i, j, lt, ll
     character(len=1) :: endtag
@@ -927,8 +936,11 @@ CONTAINS
     !
     cval = ''
     if ( eot < 0 ) then
-       ! print *, 'end of file reached, tag not found'
-       if ( present(ierr) ) ierr =-1
+       if ( .not. present(ierr) ) then
+          print *, 'end of file reached, tag not found'
+       else
+          ierr = 1
+       end if
        return
     else if ( eot == 0 ) then
        ! print *, 'tag found, no value to read on line'
@@ -953,8 +965,11 @@ CONTAINS
              lt = len_trim(tag)
              endtag = adjustl( line(j+i+1+lt:) )
              if ( endtag /= '>' ) then
-                ! print *, 'tag ',trim(tag),' not correctly closed'
-                if (present(ierr)) ierr = 1
+                if ( .not.present(ierr)) then
+                   print *, 'tag ',trim(tag),' not correctly closed'
+                else
+                   ierr = 2
+                endif
              else
                 ! end of tag found, read value (if any) and exit
                 if ( i > 1 ) cval = trim(cval) // adjustl(trim(line(j:j+i-2)))
@@ -988,16 +1003,11 @@ CONTAINS
     !
     character(len=*), intent(in) :: tag
     integer, intent(out), optional :: ierr
-    ! 0: tag found and read
-    !-1: tag not found
-    ! 1: error parsing file
-    ! 2: line too long
-    ! 3: too many levels of tags
-    !
+    ! See list of error codes in the header of this file
     integer :: stat, ntry, ll, lt, i, j, j0
+    ! stat=-1: in comment (not actually used)
     ! stat= 0: begin
-    ! stat=-1: in comment
-    ! stat=1 : tag found
+    ! stat= 1: tag found
     !
     character(len=1) :: quote
     !
@@ -1014,7 +1024,7 @@ CONTAINS
        ll = len_trim(line)
        if ( ll == maxline ) then
           print *, 'xmlr_opentag: severe error, line too long'
-          if (present(ierr)) ierr = 2
+          if (present(ierr)) ierr = 3
           return
        end if
        ! j is the current scan position
@@ -1023,20 +1033,20 @@ CONTAINS
        j0 = 1
        parse: do while ( j <= ll )
           !
-          if ( stat ==-1 ) then
-             !
-             ! scanning a comment
-             i = index(line(j:),'-->')
-             if ( i == 0 ) then
-                ! no end of comment found on this line
-                exit parse
-             else
-                ! end of comment found
-                stat = 0
-                j = j+i+3
-             end if
-             !
-          else if ( stat == 0 ) then
+          ! following case is never set and unnecessary:
+          !if ( stat ==-1 ) then
+          !   ! scanning a comment
+          !   i = index(line(j:),'-->')
+          !   if ( i == 0 ) then
+          !      ! no end of comment found on this line
+          !      exit parse
+          !   else
+          !      ! end of comment found
+          !      stat = 0
+          !      j = j+i+3
+          !   end if
+          !else if ( stat == 0 ) then
+          if ( stat == 0 ) then
              !
              ! searching for tag
              !
@@ -1067,7 +1077,7 @@ CONTAINS
                 j0= j
              else if ( line(j:j+1) == '/>' ) then
                 ! <tag ... /> found : return
-                if (present(ierr)) ierr = 0
+                if (present(ierr)) ierr =-1
                 ! eot = 0: tag with no value found
                 eot = 0
                 !
@@ -1081,7 +1091,7 @@ CONTAINS
                 nlevel = nlevel+1
                 IF ( nlevel > maxlevel ) THEN
                    print *, 'xmlr_opentag: severe error, too many levels'
-                   if (present(ierr)) ierr = 3
+                   if (present(ierr)) ierr = 4
                 else
                    open_tags(nlevel) = trim(tag)
 #if defined ( __debug )
@@ -1128,7 +1138,7 @@ CONTAINS
     !
 10  if ( stat == 0 ) then
        if ( present(ierr) ) then
-          ierr =-1
+          ierr = 1
           ! quick-and-dirty pseudo-fix to deal with tags not found:
           ! rewind and try again - will work if the desired tag is
           ! found above the current position (and nowhere else)
@@ -1139,7 +1149,7 @@ CONTAINS
        end if
     else
        print *, 'xmlr_opentag: severe parsing error'
-       if ( present(ierr) ) ierr = 1
+       if ( present(ierr) ) ierr = 2
     end if
     !
   end subroutine xmlr_opentag
@@ -1155,7 +1165,7 @@ CONTAINS
     ! 2: error parsing file
     !
     integer :: stat, ll, lt, i, j
-    ! stat=-1: in comment
+    ! stat=-1: in comment (not actually used)
     ! stat= 0: begin
     ! stat= 1: end
     !
@@ -1175,27 +1185,27 @@ CONTAINS
        ll = len_trim(line)
        if ( ll == maxline ) then
           print *, 'Fatal error: line too long'
-          if (present(ierr)) ierr = 1
+          if (present(ierr)) ierr = 2
           return
        end if
        ! j is the current scan position
        j = 1
        parse: do while ( j <= ll )
           !
-          if ( stat ==-1 ) then
-             !
-             ! scanning a comment
-             i = index(line(j:),'-->')
-             if ( i == 0 ) then
-                ! no end of comment found on this line
-                exit parse
-             else
-                ! end of comment found
-                stat = 0
-                j = j+i+3
-             end if
-             !
-          else if ( stat == 0 ) then
+          ! following case is never set and unnecessary:
+          !if ( stat ==-1 ) then
+          ! scanning a comment
+          !   i = index(line(j:),'-->')
+          !   if ( i == 0 ) then
+          ! no end of comment found on this line
+          !      exit parse
+          !   else
+          ! end of comment found
+          !      stat = 0
+          !      j = j+i+3
+          !   end if
+          !else if ( stat == 0 ) then
+          if ( stat == 0 ) then
              !
              ! searching for closing tag
              !
